'''
Created on 1. 7. 2014

@author: Vancikv
'''

from traits.api import \
    Instance, Property, \
    List, Str, Trait, Button
from traitsui.api import \
    View, Item, UItem, VGroup, HGroup, spring
from bmcs_beam.mxn.cross_section_component import \
    CrossSectionComponent
from bmcs_beam.mxn.material_types import \
    MTReinfBar, MTReinfFabric, MTMatrixMixture
from bmcs_beam.mxn.mxn_diagram import \
    MxNDiagram
from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode
from bmcs_beam.mxn.utils import \
    KeyRef


class UCPStudyElement(MxNTreeNode):
    '''Class controlling plotting options
    for an instance
    '''
    node_name = Str('<unnamed>')

    color = Trait('black', dict(black='k',
                                cyan='c',
                                green='g',
                                blue='b',
                                yellow='y',
                                magneta='m',
                                red='r')
                  )

    linestyle = Trait('solid', dict(solid='-',
                                    dashed='--',
                                    dash_dot='-.',
                                    dotted=':')
                      )

    tree_view = View(VGroup(Item('node_name', label='label'),
                            Item('linestyle'),
                            Item('color'),
                            label='Plotting options'))

    def plot(self, fig):
        ax = fig.add_subplot(1, 1, 1)
        self.content.plot_custom(ax=ax, color=self.color_, linestyle=self.linestyle_,
                                 label=self.node_name)

    def plot_ax(self, ax):
        self.content.plot_custom(ax=ax, color=self.color_, linestyle=self.linestyle_,
                                 label=self.node_name)


class UCPStudyElementMxN(UCPStudyElement):
    node_name = '<unnamed mxn diagram>'

    tree_node_list = List(Instance(MxNTreeNode))

    def _tree_node_list_default(self):
        return [MxNDiagram()]

    content = Property(depends_on='tree_node_list')

    def _get_content(self):
        return self.tree_node_list[0]

    def _set_content(self, val):
        self.tree_node_list = [val]


class UCPStudyElementFabricLaw(UCPStudyElement, CrossSectionComponent):

    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'default_fabric'
        super(UCPStudyElementFabricLaw, self).__init__(**metadata)

    node_name = '<unnamed fabric law>'
    material = KeyRef('default_fabric', db=MTReinfFabric.db)

    tree_node_list = Property(depends_on='material,material_law')

    def _get_tree_node_list(self):
        return [self.material_law_]

    tree_view = View(VGroup(Item('node_name', label='label'),
                            Item('linestyle'),
                            Item('color'),
                            label='Plotting options'),
                     VGroup(Item('material'),
                            Item('material_law'),
                            label='Law options'))


class UCPStudyElementBarLaw(UCPStudyElement, CrossSectionComponent):

    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'bar_d10'
        super(UCPStudyElementBarLaw, self).__init__(**metadata)

    node_name = '<unnamed bar law>'
    material = KeyRef('bar_d10', db=MTReinfBar.db)

    tree_node_list = Property(depends_on='material,material_law')

    def _get_tree_node_list(self):
        return [self.material_law_]

    tree_view = View(VGroup(Item('node_name', label='label'),
                            Item('linestyle'),
                            Item('color'),
                            label='Plotting options'),
                     VGroup(Item('material'),
                            Item('material_law'),
                            label='Law options'))


class UCPStudyElementMatrixLaw(UCPStudyElement, CrossSectionComponent):

    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'default_mixture'
        super(UCPStudyElementMatrixLaw, self).__init__(**metadata)

    node_name = '<unnamed matrix law>'
    material = KeyRef('default_mixture', db=MTMatrixMixture.db)

    tree_node_list = Property(depends_on='material,material_law')

    def _get_tree_node_list(self):
        return [self.material_law_]

    tree_view = View(VGroup(Item('node_name', label='label'),
                            Item('linestyle'),
                            Item('color'),
                            label='Plotting options'),
                     VGroup(Item('material'),
                            Item('material_law'),
                            label='Law options'))


class UCParametricStudy(MxNTreeNode):
    node_name = Str('Parametric study')

    element_to_add = Trait('mxndiagram', {'mxndiagram':   UCPStudyElementMxN,
                                          'fabric_law':   UCPStudyElementFabricLaw,
                                          'bar_law':   UCPStudyElementBarLaw,
                                          'matrix_law':   UCPStudyElementMatrixLaw,
                                          }
                           )

    add_element = Button('Add')

    def _add_element_fired(self):
        self.append_node(self.element_to_add_())

    tree_view = View(HGroup(UItem('element_to_add', springy=True),
                            UItem('add_element')),
                     spring
                     )

    tree_node_list = List(Instance(MxNTreeNode))

    def _tree_node_list_default(self):
        return []

    def plot(self, fig):
        ax = fig.add_subplot(1, 1, 1)
        for node in self.tree_node_list:
            node.plot_ax(ax=ax)
        ax.legend()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 26, 2010 by: rch

from traits.api import \
    HasTraits, Float, Property, cached_property, \
    Instance, List, on_trait_change, Int, Tuple, Bool, \
    DelegatesTo, Event, Str, Button, Dict, Array, Any, \
    implements

from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor, Handler

class ToDo( HasTraits ):
    todo_string = Str( 'This is on a todo list' )
    traits_view = View( Item( 'todo_string', show_label = False, springy = True,
                              style = 'readonly' ),
                        width = 0.2,
                        height = 0.2,
                        buttons = ['OK'] )





'''
Created on Apr 22, 2010

@author: alexander
'''
from traits.api import \
    HasTraits, Directory, List, Int, Float, Any, \
    on_trait_change, File, Constant, Instance, Trait, \
    Array, Str, Property, cached_property, WeakRef, \
    Dict, Button, Bool, Enum, Event, implements, \
    DelegatesTo, Regex

from traitsui.api import \
    View, Item, DirectoryEditor, TabularEditor, HSplit, VGroup, \
    TableEditor, EnumEditor, Handler, FileEditor, VSplit, Group, \
    InstanceEditor, HGroup, Spring
    

from etsproxy.units.ui.quantity_view import \
    QuantityView
    
from etsproxy.units import \
    *

from etsproxy.units.convert import \
    convert 

from etsproxy.units.quantity import \
    Quantity

from etsproxy.traits.ui.table_column import \
    ObjectColumn
    
from etsproxy.traits.ui.menu import \
    OKButton, CancelButton
    
from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter

from numpy import \
    array, fabs, where, copy, ones, linspace, ones_like, hstack, arange

from etsproxy.traits.ui.table_filter \
    import EvalFilterTemplate, MenuFilterTemplate, RuleFilterTemplate, \
           EvalTableFilter

from etsproxy.traits.ui.file_dialog  \
    import open_file, FileInfo, TextInfo, ImageInfo

from traitsui.api \
    import View,TabularEditor
    
from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter

from etsproxy.units.unit \
    import *

from util.traits.ui.item import \
    Item


#---------------------------------------------------------------------

def get_unit( trait ):
    '''Return the string stored in the trait metadata 
    for key 'unit'. If unspecified return 'None'.
    '''
    dict = trait.__dict__
    if 'unit' in dict:
        return dict['_metadata']['unit'] 

def get_convert_fact( unit_old, unit_new  ):
    '''Returns a factor for converting 'unit_old' to 'unit_new' 
    unit_old  - given unit to be converted
    unit_new - converted unit (target unit) 
    '''
    if unit_old == None:  
        return 1.
    else:
        m_dict = {'m':1, 'dm':10, 'cm':100, 'mm':1000,}
        return float(m_dict[ unit_new ]) / float( m_dict[ unit_old ] ) 

def unit2m( trait ):
    '''Convert trait to unit 'm'. Multiply value with a factor
    based on the unit specified in the trait's '_metadata'  
    '''
    unit = get_unit( trait )
    fact = get_convert_fact( unit, 'm')
    value = trait.__dict__['default_value']
    return fact * value


class Bcl( HasTraits ):
    '''Example class with a trait with metadata 'unit'
    '''
    #------------------------------
    # convert unit
    #------------------------------
    bb = Int(3, unit = 'cm')
    cc = Int(3 )


class Acl( HasTraits ):
    '''Example class with a trait with metadata 'unit'
    '''
    #------------------------------
    # convert unit
    #------------------------------
    b = Int(3, unit = 'cm')
    c = Int(3 )
    u = get_unit( b )
    f = get_convert_fact( u, 'm' )
    b_m = unit2m(b)

    #------------------------------
    # test indexing of Array object
    #------------------------------
#    B = Property( Array, unit = 'm' )
#    def _get_B(self):
#        return array([1,2,3])

#    bbb = arange(2)

    B = Array( unit = 'm' )
    def _B_default(self):
        print('default set')
        return array([9,9])    

    def ppp(self):
        print(self.B[0])
        return 1
    
    
#    B.default_value = array([22])    
    
    #------------------------------
    # set attributes
    #------------------------------
    factor_list = ['WA_1', 'WA_2', 'WA_3']
    units = ['s', 'kN', 'mm']
    processed_data_array = 1.0 * arange(9).reshape(3,3)

    def set_attr_type_and_unit(self):
        for i, factor in enumerate( self.factor_list ):
            setattr( self, factor, Array( unit = self.units[i] ))
            #attr = getattr( self, factor )
            #attr.default_value = self.processed_data_array[:,i] 
            setattr( self, factor+'.default_value', self.processed_data_array[:,i] )
            
    BI = Instance(Bcl)
    def _BI_default(self):
        return Bcl()
    
    traits_view = View(
                       Item('b', show_label = True,),
                       Item('c', show_label = True,),
                       Item('BI@', show_label = False,),
                       Item('BI@', show_label = True,),
                       scrollable = True,
                       resizable = True,
                       height = 0.4,
                       width = 0.4,
                       )

#---------------------------------

if __name__ == '__main__':

    A = Acl()
    A.set_attr_type_and_unit()
#    A.configure_traits()

    print('WA_1', A.WA_1)
    dict = A.WA_1.__dict__ 
    print('dict', dict)   
    
    B = A.B
    print('B',B[0])
    
    B = A.ppp()




'''
Created on 15. 2. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoCirc

from bmcs_beam.mxn.reinf_layout import \
    RLCBar, RLCTexLayer

import numpy as np

def test_cross_section_mn():
    '''Test the moment and normal force calculated for a cross section.
    Circular cross section with mixed reinforcement.
    '''

    radius = 0.3
    ge = MCSGeoCirc(radius=0.3)
    '''Cross section geometry
    '''

    n_bars = 10
    radius_bars = 0.26
    bar_area = 0.0002
    '''Specifications of bar reinforcement
    '''

    fi_bar_arr = np.arange(0, 2 * np.pi, 2 * np.pi / n_bars, dtype=float)
    x_bar_arr = np.cos(fi_bar_arr) * radius_bars + radius
    z_bar_arr = radius - np.sin(fi_bar_arr) * radius_bars
    '''Positions of bars divided equally across the circumference
    '''

    bar_lst = []
    '''List of bars
    '''
    for i in range(n_bars):
        bar_lst.append(RLCBar(x=x_bar_arr[i], z=z_bar_arr[i], material='bar_d10'))
    bar_lst[0].material_.area = bar_area

    tl1 = RLCTexLayer(z_coord=0.15, material='default_fabric', material_law='fbm')
    tl2 = RLCTexLayer(z_coord=0.16, material='default_fabric', material_law='fbm')
    '''Two layers of textile reinforcement
    '''

    cs = CrossSection(reinf=[tl1, tl2] + bar_lst,
                             matrix_cs=MatrixCrossSection(geo=ge,
                                         n_cj=20, material='default_mixture',
                                         material_law='constant'),
                             eps_lo=0.002,
                             eps_up=-0.0033,
                             )

    tl1.material_law_.set(sig_tex_u=1216., eps_u=0.014, m=0.5)
    tl1.material_.set(s_0=0.0247, A_roving=0.5)

    assert np.allclose([cs.M, cs.N], [1236.3684520623658, -9300.9390846971837])
    bar_lst[0].material_.area = 0.0004
    assert np.allclose([cs.M, cs.N], [1349.182793672938, -9534.1983257666307])
    ge.radius = 0.33
    assert np.allclose([cs.M, cs.N], [1702.868895888816, -11299.173023898555])
    cs.eps_lo = 0.004
    assert np.allclose([cs.M, cs.N], [1626.2468647131818, -6796.3926837363797])

if __name__ == '__main__':
    test_cross_section_mn()



'''
Created on 30.08.2016

@author: Yingxiong
'''
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider


E = 100.
eps_cu = -0.2  # the minimum matrix strain
# material law - matrix
Sig = lambda eps: E * eps * (eps >= eps_cu) * (eps <= 0.0)
# material law - reinforcement
eps_u_r = 0.2
Sig_reinf = lambda eps: 500. * eps * \
    (eps >= -eps_u_r * 1.05) * (eps <= eps_u_r * 1.05)

reinf_y_coord = np.array([6., 7., 4., 2., 1, 9])
reinf_area = np.array([0.1, 0.1, 0.1, 0.1, 0.1, 0.1])


height = 10.
gravity_center = height / 2.
width = 2.

n_ip = 10000
# discretization of the matrix
y_coord = np.linspace(0, height, n_ip)

# upper strain
eps_upper = -0.1
# lower strain
eps_lower = 0.05


def get_NM(eps_lower, eps_upper):
    # matrix strain array
    strain_y = np.interp(y_coord, [0, height], [eps_lower, eps_upper])

    # matirx stress array
    stress_y = Sig(strain_y)

    # reinf strain
    strain_r = np.interp(reinf_y_coord, [0, height], [eps_lower, eps_upper])

    # reinf stress
    stress_r = Sig_reinf(strain_r)

    # normal force - matrix
    N_m = np.trapz(stress_y * width, y_coord)
    # normal force - reinforcement
    N_r = np.sum(stress_r * reinf_area)
    N = N_m + N_r

    # moment - matrix
    M_m = np.trapz(stress_y * width * (y_coord - gravity_center), y_coord)
    # moment - reinforcement
    M_r = np.sum(stress_r * reinf_area * (reinf_y_coord - gravity_center))
    M = M_m + M_r

    return N, M, strain_y, stress_y, strain_r, stress_r

# get the mxn diagram

# Convert the strain in the lowest reinforcement layer at failure
# to the strain at the bottom of the cross section


def convert_eps_u_2_lo(eps_up, eps_u_r, y_coord_r):
    # eps_u_r -- the maximum strain of the corresponding reinforcement
    h = height
    eps_lo = eps_up + (eps_u_r - eps_up) / (h - y_coord_r) * h
    return eps_lo

eps_lo_arr = convert_eps_u_2_lo(-0.1, eps_u_r, reinf_y_coord)
env_reinf_idx = np.argmin(eps_lo_arr)

eps_ccu = 0.8 * eps_cu


n_eps = 20
reinforced = True
if reinforced:
    eps_t_lo = convert_eps_u_2_lo(
        eps_cu, eps_u_r, reinf_y_coord[env_reinf_idx])
    eps_t_lo_0 = convert_eps_u_2_lo(0., eps_u_r, reinf_y_coord[env_reinf_idx])
    eps_t_u = eps_u_r

    # Strain arrays for the lower rim
    eps_cc_0 = np.linspace(eps_ccu, 0., n_eps)
    eps_0_tlo = np.linspace(0., eps_t_lo, n_eps)
    eps_tlo_tlo0 = np.linspace(eps_t_lo, eps_t_lo_0, n_eps)
    eps_tlo0_tu = np.linspace(eps_t_lo_0, eps_t_u, n_eps)

    # Strain arrays for the upper rim
    eps_cc_c = np.linspace(eps_ccu, eps_cu, n_eps)
    eps_c_const = eps_cu * np.ones_like(eps_cc_0)
    eps_c_0 = np.linspace(eps_cu, 0., n_eps)
    eps_0_tu = np.linspace(0., eps_t_u, n_eps)

    eps1 = np.vstack([eps_cc_0, eps_cc_c])
    eps2 = np.vstack([eps_0_tlo, eps_c_const])
    eps3 = np.vstack([eps_tlo_tlo0, eps_c_0])
    eps4 = np.vstack([eps_tlo0_tu, eps_0_tu])

    eps_arr = np.hstack([eps1, eps2, eps3, eps4])
else:
    # Strain arrays for the lower rim
    eps_cc_0 = np.linspace(eps_ccu, 0., n_eps)
    eps_00 = np.zeros_like(eps_cc_0)

    # Strain arrays for the upper rim
    eps_cc_c = np.linspace(eps_ccu, eps_cu, n_eps)
    eps_c_0 = np.linspace(eps_cu, 0., n_eps)

    eps1 = np.vstack([eps_cc_0, eps_cc_c])
    eps2 = np.vstack([eps_00, eps_c_0])

    eps_arr = np.hstack([eps1, eps2])

# get the m-n envelop
M_arr = np.zeros_like(eps_arr[0])
N_arr = np.zeros_like(eps_arr[0])
for i in np.arange(len(eps_arr[0])):

    result = get_NM(eps_arr[0][i], eps_arr[1][i])

    N_arr[i] = result[0]
    M_arr[i] = result[1]
#     print result[1]

# print np.where(np.abs(M_arr + 78.94480586) < 1.)

N, M, strain_y, stress_y, strain_r, stress_r = get_NM(
    eps_arr[0][0], eps_arr[1][0])

ax1 = plt.subplot(221)
ax1.plot(strain_y, y_coord)
ax1.plot((0, 0), (0, height))
for i, strain in enumerate(strain_r):
    ax1.plot((strain, 0), (reinf_y_coord[i], reinf_y_coord[i]), '--')

ax2 = plt.subplot(222)
ax2.plot(stress_y, y_coord)
ax2.plot((0, 0), (0, height))
for i, stress in enumerate(stress_r):
    ax2.plot((stress, 0), (reinf_y_coord[i], reinf_y_coord[i]), '--')

ax3 = plt.subplot(223)
ax3.plot(-eps_arr, [0, height], 'k-')
l, = ax3.plot(-eps_arr.T[0], [0, height], 'r')

ax4 = plt.subplot(224)
ax4.plot(-M_arr, -N_arr, lw=2)
p, = ax4.plot(-M_arr[0], -N_arr[0], 'ro')
ax4.axhline(0, color='k')
ax4.axvline(0, color='k')

ax = plt.axes([0.10, 0.015, 0.80, 0.02])

step_slider = Slider(
    ax, 'step', 0.0, len(eps_arr[0]) - 1, valinit=0, valfmt='%1.0f')


def update(val):
    step = step_slider.val
    step = int(round(step))
    l.set_xdata(-eps_arr.T[step])
    p.set_data(-M_arr[step], -N_arr[step])
    N, M, strain_y, stress_y, strain_r, stress_r = get_NM(
        eps_arr[0][step], eps_arr[1][step])
    ax1.cla()
    ax1.plot(strain_y, y_coord)
    ax1.plot((0, 0), (0, height))
    for i, strain in enumerate(strain_r):
        ax1.plot((strain, 0), (reinf_y_coord[i], reinf_y_coord[i]), '--')
    ax2.cla()
    ax2.plot(stress_y, y_coord)
    ax2.plot((0, 0), (0, height))
    for i, stress in enumerate(stress_r):
        ax2.plot((stress, 0), (reinf_y_coord[i], reinf_y_coord[i]), '--')


step_slider.on_changed(update)

plt.show()



#!/usr/bin/env python
""" The entry point for an Envisage application. """

# Standard library imports.
import sys
import os.path
import logging

# Enthought library imports.
from etsproxy.mayavi.plugins.app import get_plugins, setup_logger
from traits.api import List
from etsproxy.envisage.api import Plugin, ServiceOffer
from etsproxy.envisage.ui.workbench.api import WorkbenchApplication
from etsproxy.pyface.workbench.api import Perspective, PerspectiveItem

###############################################################################
# `ProModPlugin` class.
###############################################################################
class ProModPlugin(Plugin):

    # Extension points we contribute to.
    SERVICE_OFFERS = 'enthought.envisage.ui.workbench.service_offers'

    # The plugin's unique identifier.
    id = 'ProMod.ProMod'

    # The plugin's name (suitable for displaying to the user).
    name = 'ProMod'

    # Services we contribute.
    service_offers = List(contributes_to = SERVICE_OFFERS)
    
    ######################################################################
    # Private methods.
    def _service_offers_default(self):
        """ Trait initializer. """
        promod_service_offer = ServiceOffer(
            protocol = 'promod.plugins.promod_service.ProModService',
            factory = 'promod.plugins.promod_service.ProModService'
        )

        return [promod_service_offer]



'''
Created on Mar 6, 2012

@author: rch
'''

import fnmatch
import os.path

def replace_string_in_files(path,
                            replace_dict,
                            file_ext = '.pickle'
                            ):
    '''Walk through all files within the subdirectories of `path'
    and perform the replacement of strings specifified in `replace_dict\
    in all files ending with `file_ext'
    '''
    selected_files = []
    for path, dirs, files in os.walk(path):
        for f in fnmatch.filter(files, '*' + file_ext):
            selected_files.append(os.path.join(path, f))

    for file_name in selected_files:
        print('replacement applied to', file_name)
        fid = open(file_name, "r")
        text = fid.read()
        fid.close()
        # write the replacements back into the file.
        fid = open(file_name, "w")
        for old, new in list(replace_dict.items()):
            text = text.replace(old, new)
        fid.write(text)
        fid.close()
        print('finished')

def migrate_classes(migration_table):
    '''Walk through all files within the simdb storage'
    and perform the replacement of strings specifified in `replace_dict\
    in all files ending with pickle'
    '''
    from matresdev.db.simdb import SimDB
    simdb = SimDB()    

    replace_string_in_files(simdb.exdata_dir,
                            migration_table, '.pickle')
    replace_string_in_files(simdb.matdata_dir,
                            migration_table, '.pickle')
    replace_string_in_files(simdb.exdata_dir,
                            migration_table, 'ex_type.cls')
                    
if __name__ == '__main__':

    from matresdev.db.simdb import SimDB
    simdb = SimDB()

    migration_table = {'promod.matdb.trc' : 'matresdev.db.matdb.trc',

                       'promod.exdb.ex_composite_tensile_test' : 'quaducom.devproc.tensile_test.dog_bone.exp_tt_db',
                       'ExCompositeTensileTest' : 'ExpTensileTestDogBone',
#                       'ExCompositeTensileTest' : 'ExpDogBoneTensileTest',
#                       'promod.exdb.ex_bending_test' : 'quaducom.devproc.bt.p3.exp_bt_3pt',
                       'promod.exdb.ex_bending_test' : 'quaducom.devproc.bending_test.three_point.exp_bt_3pt',
#                       'ExBendingTest' : 'ExpBendingTest3Pt',
                       'ExBendingTest' : 'ExpBT3Pt',

#                       'promod.exdb.ex_plate_test' : 'quaducom.devproc.st.exp_st',
                       'promod.exdb.ex_plate_test' : 'quaducom.devproc.slab_test.exp_st',
#                       'ExPlateTest' : 'ExpSlabTest',
                       'ExPlateTest' : 'ExpST',
 
                       'exp_dbtt':'exp_tt_db',
#                       'promod.exdb.ex_composite_tensile_test' : 'quaducom.devproc.tt.dbtt.exp_dbtt',
#                       'quaducom.devproc.tt.dbtt' : 'quaducom.devproc.tensile_test.dog_bone',
                       'ExpDogBoneTensileTest' : 'ExpTTDB',
                       # also replace class name in the file "exp_tt_db.py" 

#                       'quaducom.devproc.bt.p4' : 'quaducom.devproc.bending_test.four_point',
#                       'quaducom.devproc.bt.p3' : 'quaducom.devproc.bending_test.three_point',
#                       'ExpBendingTest3Pt' : 'ExpBendingTestThreePoint',
#                       'ExpBendingTest4Pt' : 'ExpBendingTestFourPoint',

                       'quaducom.devproc.st' : 'quaducom.devproc.slab_test',
                       'quaducom.devproc.dt' : 'quaducom.devproc.disk_test',

                        'DiskTestSetup':'SimDT',
                        'ExpSlabTest':'ExpST',
                        'SimSlabTest':'SimpST',

                        'quaducom.devproc.tt.bctt' : 'quaducom.devproc.tensile_test.buttstrap_clamping',
#                        'ButtstrapClamping': 'ExpTTBC',
                        'ExpTensileTestButtstrapClamping':'ExpTTBC'

#                        'SimTreePointBending' : 'SimBT3PT'
#                        'SimTreePointBendingDB':'SimBT3PTDB'

    } 

    migration_table = {'enthought.traits' : 'traits'}

    migrate_classes(migration_table)



'''
Created on Jun 23, 2010

@author: alexander
'''

from traits.api import \
    Float, Instance, Array, Property, cached_property, \
    Int

from traitsui.api import \
    View, Item, Group, \
    InstanceEditor, VGroup

import numpy as np

from scipy.optimize import fsolve

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

from bmcs_beam.mxn.reinf_laws import \
    ReinfLawBase

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from .matresdev.db.simdb import SimDB
simdb = SimDB()

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

class ECBCalib(MxNTreeNode):

    # rupture moment and normal force measured in the calibration experiment
    # (three point bending test)
    #
    Mu = Float(3.5, calib_input=True)  # [kNm]
    Nu = Float(0.0, calib_input=True)  # [kN]

    #===========================================================================
    # Cross Section Specification (Geometry and Layout)
    #===========================================================================

    cs = Instance(CrossSection)
    def _cs_default(self):
        return CrossSection(reinf=[RLCTexUniform(n_layers=12, material='default_fabric')],
                               matrix_cs=MatrixCrossSection(geo=MCSGeoRect(width=0.2,
                                        height=0.06), n_cj=20, material='default_mixture',
                                                            material_law='constant'))

    u0 = Property(Array(float), depends_on='cs.changed')
    '''Construct the initial vector.
    '''
    @cached_property
    def _get_u0(self):
        u0 = self.cs.reinf_components_with_state[0].material_law_.u0

        eps_up = -self.cs.matrix_cs.material_law_.eps_c_u
        eps_lo = self.cs.reinf_components_with_state[0].convert_eps_u_2_lo(eps_up=eps_up)

        print('eps_up', eps_up)
        print('eps_lo', eps_lo)

        return np.array([eps_lo, u0[1] ], dtype='float')

    # iteration counter
    #
    n = Int(0)
    def get_lack_of_fit(self, u):
        '''Return the difference between 'N_external' and 'N_internal' as well as 'M_external' and 'M_internal'
        N_c (=compressive force of the compressive zone of the concrete)
        N_t (=total tensile force of the reinforcement layers)
        '''

        print('--------------------iteration', self.n, '------------------------')
        self.n += 1
        # set iteration counter
        #
        eps_up = -self.cs.matrix_cs.material_.eps_c_u
        eps_lo = u[0]

        self.cs.set(eps_lo=eps_lo, eps_up=eps_up)

        eps_tex_u = self.cs.reinf_components_with_state[0].converted_eps_lo_2_u
        self.cs.reinf_components_with_state[0].material_law_.set_cparams(eps_tex_u, u[1])

        N_internal = self.cs.N
        M_internal = self.cs.M

        d_N = N_internal - self.Nu
        d_M = M_internal - self.Mu

        return np.array([ d_N, d_M ], dtype=float)

    u_sol = Property(Array(Float), depends_on='cs.changed,+calib_input')
    '''Solution vector returned by 'fit_response'.'''
    @cached_property
    def _get_u_sol(self):
        '''iterate 'eps_t' such that the lack of fit between the calculated
        normal forces in the tensile reinforcement and the compressive zone (concrete)
        is smaller then 'xtol' defined in function 'brentq'.
        NOTE: the method 'get_lack_of_fit' returns the relative error.
        '''

        # use scipy-functionality to get the iterated value of 'eps_t'
        # NOTE: get_lack_of_fit must have a sign change as a requirement
        # for the function call 'brentq' to work property.

        # The method brentq has optional arguments such as
        #   'xtol'    - absolut error (default value = 1.0e-12)
        #   'rtol'    - relative error (not supported at the time)
        #   'maxiter' - maximum numbers of iterations used
        #
        return fsolve(self.get_lack_of_fit, self.u0, xtol=1.0e-5)

    calibrated_ecb_law = Property(Instance(ReinfLawBase), depends_on='cs.changed,+calib_input')
    '''Calibrated ecbl_mfn
    '''
    @cached_property
    def _get_calibrated_ecb_law(self):
        print('NEW CALIBRATION')
        self.cs.eps_lo = self.u_sol[0]
        eps_tex_u = self.cs.reinf_components_with_state[0].converted_eps_lo_2_u
        self.cs.reinf_components_with_state[0].material_law_.set_cparams(eps_tex_u, self.u_sol[1])
        self.n = 0
        self.cs.reinf_components_with_state[0].material_.save()
        return self.cs.reinf_components_with_state[0].material_law_

    ecb_law = Property(Instance(ReinfLawBase))
    '''Not calibrated law
    '''
    def _get_ecb_law(self):
        return self.cs.reinf_components_with_state[0].material_law_

    #===========================================================================
    # Visualisation related attributes
    #===========================================================================

    def plot(self, fig):
        self.calibrated_ecb_law.plot(fig)

    node_name = 'ECB law calibration'

    tree_node_list = Property
    @cached_property
    def _get_tree_node_list(self):
        return [self.cs]

    traits_view = View(
                Item('Mu'),
                Item('Nu'),
                buttons=['OK', 'Cancel'],
                )

    tree_view = View(VGroup(
                Group(
                Item('Mu'),
                Item('Nu'),
                ),
                Group(
                Item('ecb_law',
                     editor=InstanceEditor(editable=True),
                     style='custom',
                     show_label=False),
                label='Effective crack bridge law'
                ),
                ),
                buttons=['OK', 'Cancel'],
                )

if __name__ == '__main__':

    #------------------------------------------------
    # 1) CALIBRATION:
    # get 'eps_t' and the parameter of the effective
    # crack bridge function 'var_a' for a given 'eps_c_u'
    #------------------------------------------------
    #

    print('\n')
    print('setup ECBLCalib')
    print('\n')

    ec = ECBCalib(Mu=3.49)




#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on May 29, 2009 by: rch

from traits.api import \
    Float, Instance, Array, Int, Property, cached_property, on_trait_change, Bool, \
    HasTraits, File, Event, Str

from traitsui.api import \
    View, Item, FileEditor, HSplit, Group, VSplit, \
    Handler

from etsproxy.traits.ui.menu import \
    Action, CloseAction, HelpAction, Menu, \
    MenuBar, NoButtons, Separator, ToolBar, CancelButton, OKButton

from etsproxy.pyface.api import ImageResource

from ibvpy.mats.mats_explore import MATSExplore
from ibvpy.mats.mats2D.mats2D_explore import MATS2DExplore
from numpy import copy, array, hstack, loadtxt, savetxt
from bmcs_beam.mxn.mfn import MFnLineArray

from scipy.optimize import brentq, newton, fsolve, brenth
from os.path import join
from ibvpy.core.tloop import TLoop, TLine
from ibvpy.core.scontext import SContext
from ibvpy.core.tstepper import TStepper

from matresdev.db.exdb.ex_run import ExRun

data_file_editor = FileEditor(filter=['*.DAT'])

from util.traits.editors.mpl_figure_editor import MPLFigureEditor
from matplotlib.figure import Figure
import pickle
from copy import copy

from matresdev.db.simdb import SimDB
simdb = SimDB()

import os

import pylab as p
import numpy as np

# ---------------------------------------------------
# Calibration controller
# ---------------------------------------------------

class MATSCalibDamageFnController(Handler):
    '''Handle the dynamic interaction with the calibrator.
    '''

    #---------------------------------------------------------------------------
    # Public Controller interface
    #---------------------------------------------------------------------------
    def run_calibration(self, ui_info):

        calibrator = ui_info.object

        calibrator.init()
        fit_response = calibrator.fit_response()

# ---------------------------------------------------
# Calibrator of the damage function from uniaxial test:
# ---------------------------------------------------

class MATSCalibDamageFn(MATSExplore):
    '''
    Fitting algorithm for the damage function of the
    quasi-ductile anisotropic material model.

    The algorithm uses the TLoop instance to proceed step
    by step with the computation. The value of the damage function
    for the time step t_n is identified iteratively by adjusting
    the values and evaluating the corresponding equilibrated stresses.

    The control parameters of the algorithm are:

    @param step_size: time step for fitting the damage parameter.
    @param tmax: end time for fitting, it might be also be set implicitly
    for integrity = 1 - full damage of the material.
    '''

    # store the fitted 'PhiFn' in the data base, i.e. 'CCSUniteCell'
    store_fitted_phi_fn = Bool(True)

    # default settings are overloaded with settings specified in 'ec_config'

    max_eps = Property(Float)
    def _get_max_eps(self):
#        return 0.007  # set explicit value when calibration is aborted (mean value of strain)
        return self.mfn_line_array_target.xdata[-1]

    n_steps = Int(1)

    log = Bool(False)

    # TLine parameter
    #
    KMAX = Int
    tolerance = Float
    RESETMAX = Float

    step_size = Property(Float, depends_on='max_eps,n_steps')
    @cached_property
    def _get_step_size(self):
        print('step_size = ', self.max_eps / self.n_steps)
        return self.max_eps / self.n_steps


    def run_through(self):
        '''Run the computation without fitting from the start to the end
        '''
        self.tloop.tline.max = self.tmax
        self.tloop.tline.step = self.step_size
        self.tloop.eval()
        print('ending time', self.tloop.t_n1)
        # show the response

    def run_step_by_step(self):
        '''Run the computation step by step from the start to the end
        '''
        n_steps = int(self.tmax / self.step_size)
        self.tloop.tline.step = self.step_size
        current_time = 0.
        tmax = 0.
        for i in range(n_steps):
            print('STEP', i)
            self.run_one_step()

    def run_trial_step(self):
        '''Run the computation one step starting from the
        current time t_n to iterate the value for phi_new
        which gives a fit with macroscopic stress curve.
        NOTE: The trial step does not update 'U_n' or 't_n'!
        '''
        if self.log:
            print('--------- run trial step: --------- ')
        if len(self.tloop.U_n) == 0:
            current_U_n = self.tloop.tstepper.new_cntl_var()
            print('U_n = None: tloop.tstepper.new_cntl_var()', self.tloop.tstepper.new_cntl_var())
        else:
            current_U_n = self.tloop.U_n[:]
        current_time = self.tloop.t_n
        self.run_one_step()

        # reset the current time back
        self.tloop.t_n = current_time
        self.tloop.U_n[:] = current_U_n[:]
        if self.log:
            print('--------- end of trial step --------- ')

        self.tloop.tstepper.sctx.update_state_on = False

    def run_one_step(self):
        '''Run the computation one step starting from the
        current time t_n with the iterated value for phi_new
        in order to update TLoop and save the new phi value in
        the array ydata of PhiFnGeneral
        NOTE: The calculated step does update 'U_n' or 't_n'!
        '''
        self.tloop.tline.step = self.step_size
        current_time = self.tloop.t_n
        tmax = current_time + self.step_size
        self.tloop.tline.max = tmax
        self.tloop.eval()
        self.update_e_max_value_new = True

    #--------------------------------------------------
    # Data source for calibration within simdb
    #--------------------------------------------------

    ex_run = Instance(ExRun)

    composite_tensile_test = Property
    def _get_composite_tensile_test(self):
        return self.ex_run.ex_type

    composite_cross_section = Property
    def _get_composite_cross_section(self):
        return self.composite_tensile_test.ccs

    def get_target_data_exdb_tensile_test(self):
        '''Use the data from the ExDB
        '''
        ctt = self.composite_tensile_test

        # save 'sig_eps_arr' in directory "/simdb/simdata/mats_calib_damage_fn"
        simdata_dir = os.path.join(simdb.simdata_dir, 'mats_calib_damage_fn')
        if os.path.isdir(simdata_dir) == False:
            os.makedirs(simdata_dir)
        ctt_key = str(self.composite_tensile_test.key)
        filename = os.path.join(simdata_dir, 'eps-sig-arr_' + ctt_key + '.csv')

        xdata, ydata = ctt.eps_c_interpolated_smoothed[:, None], ctt.sig_c_interpolated_smoothed[:, None]
        eps_sig_arr = np.hstack([xdata, ydata])
        print('eps_sig_arr')
        np.savetxt(filename, eps_sig_arr, delimiter=';')
        print('eps-sig-data saved to file %s' % (filename))

        return ctt.eps_c_interpolated_smoothed, ctt.sig_c_interpolated_smoothed  # smoothed data without jumps with interpolated starting point in the origin
#        return ctt.eps_c_interpolated, ctt.sig_c_interpolated  # original data without jumps with interpolated starting point in the origin
#        return ctt.eps_ironed, ctt.sig_c_ironed  # original data without smoothing (without jumps)
#        return ctt.eps_smooth, ctt.sig_c_smooth #smoothed data


    #--------------------------------------------------
    # interpolation function for fitting data:
    #--------------------------------------------------
    mfn_line_array_target = Property(Instance(MFnLineArray),
                                      depends_on='ex_run')
    @cached_property
    def _get_mfn_line_array_target(self):
        xdata, ydata = self.get_target_data_exdb_tensile_test()
        print('xdata[-1]', xdata[-1])
        return MFnLineArray(xdata=xdata, ydata=ydata)

    fitted_phi_fn = Instance(MFnLineArray)

    #---------------------------------------------------------------
    # PLOT OBJECT
    #-------------------------------------------------------------------
    figure = Instance(Figure)
    def _figure_default(self):
        figure = Figure(facecolor='white')
        figure.add_axes([0.12, 0.13, 0.85, 0.74])
        return figure

    data_changed = Event

    def init(self):
        #--------------------------------------------------
        # for fitting use 'General'-function for 'phi_fn':
        #--------------------------------------------------
        # The value pair for the piecewise linear definition
        # of 'phi_fn' value consists of current strain and the
        # iterated 'phi_value'. The microplanes with a lower
        # microplane strain level use an interpolated value
        # for 'phi'
        self.fitted_phi_fn = self.dim.mats_eval.phi_fn.mfn
        self.fitted_phi_fn.xdata = [0]
        self.fitted_phi_fn.ydata = [1]
        self.fitted_phi_fn.data_changed = True
        # initialize TLoop parameters:
        self.tloop.setup()
        self.tloop.tstepper.sctx.mats_state_array[:] = 0.0
        self.tloop.U_n[:] = 0.0
        self.tloop.rtrace_mngr.clear()
        self.tloop.verbose_iteration = False
        self.tloop.verbose_load_step = False
        self.tloop.verbose_time = False
        # set TLine parameters
        self.tloop.tline.KMAX = self.KMAX
        self.tloop.tline.tolerance = self.tolerance
        self.tloop.tline.RESETMAX = self.RESETMAX

    # store trial step data in the lists if trial steps are to be stored
    # for the plotting method 'plot_trail_steps'
    #
    rec_trial_steps = True
    phi_trial_list_i = []
    sig_trial_list_i = []
    phi_trial_list_n = []
    sig_trial_list_n = []

    def get_lack_of_fit(self, phi_trial):
        '''Return the difference between the macroscopic stress calculated
        based on the value of phi_trial (damage at the next step) and the
        macroscopic stress defined as target data (=fitting curve)
        '''
        if self.log:
            print('\n')
            print("#'get_lack_of_fit' for the trial value # START")
            print('    phi_trial    = ', phi_trial)

        # value of the principle macroscopic strain corresponds to control variable
        current_time = self.tloop.t_n

        if self.log:
            print('    current_time = ', current_time)
            print('    step_size    = ', self.step_size)

        # ------------------------------------
        # add new pair in fitted_phi_fn
        # ------------------------------------
        # consisting of 'e_max_value_new' and 'phi_trial'
        x = hstack([ self.fitted_phi_fn.xdata[:], current_time + self.step_size ])
        y = hstack([ self.fitted_phi_fn.ydata[:], phi_trial ])
        self.fitted_phi_fn.set(xdata=x, ydata=y)
        self.fitted_phi_fn.data_changed = True

        # ------------------------------------
        # get state array before trial:
        # ------------------------------------
        mats_state_array_old = copy(self.tloop.tstepper.sctx.mats_state_array)

        # ------------------------------------
        # run trial step:
        # ------------------------------------
        if self.log:
            print('    reset current_U_n   =', self.tloop.U_n)
            print('CURRENT PHI', self.dim.mats_eval.phi_fn.mfn.ydata)
        # try the next equilibrium
        self.run_trial_step()

        # ------------------------------------
        # reset mats_state_array:
        # ------------------------------------
        # Note: the material state array (i.e. the maximum microstrains) are
        # updated within the iterations of each trial step, therefore a reset
        # is necessary in order to start each trial step with the same state variables
        self.tloop.tstepper.sctx.mats_state_array[:] = mats_state_array_old[:]
        if self.log:
            print('    reset state array')

        # ------------------------------------
        # remove trial value in fitted_phi_fn
        # ------------------------------------
        x = self.fitted_phi_fn.xdata[:-1]
        y = self.fitted_phi_fn.ydata[:-1]
        self.fitted_phi_fn.set(xdata=x, ydata=y)
        self.fitted_phi_fn.data_changed = True

        # ------------------------------------
        # get the lack of fit
        # ------------------------------------
        # get calculated value for 'sig_app' based on the current value of 'phi_trial':
        # and evaluate the difference between the obtained stress and the measured response
        self.tloop.rtrace_mngr.rtrace_bound_list[0].redraw()
        sig_app_trial = self.tloop.rtrace_mngr.rtrace_bound_list[0].trace.ydata[-1]
        # get corresponding value from the target data:
        sig_app_target = self.mfn_line_array_target.get_value(current_time + self.step_size)
        # absolut error:
        lack_of_fit_absolut = sig_app_trial - sig_app_target
        # relative error:
        lack_of_fit_relative = lack_of_fit_absolut / sig_app_target

        if self.log:
            print('    sig_app_trial ', sig_app_trial)
            print('    sig_app_target', sig_app_target)
            print('    lack_of_fit_absolute  ', lack_of_fit_absolut)
            print('    lack_of_fit_relative  ', lack_of_fit_relative)
            print('# get_lack_of_fit # END ')

        if self.rec_trial_steps:
            # store all trial values of 'phi_trail' and 'sig_app_trail' within each iteration to a global list
            #
            self.phi_trial_list_i.append(phi_trial)
            self.sig_trial_list_i.append(sig_app_trial)

        return lack_of_fit_relative

    param_key = Str('')

    def fit_response(self):
        '''iterate phi_trial in each incremental step such that the
        lack of fit between the calculated stress and the target
        curve is smaller then xtol defined in function 'brentq'.
        NOTE: the method 'get_lack_of_fit' returns the relative error.
        '''

        self.tloop.reset()

        phi_old = 1.0

        # map the array dimensions to the plot axes
        #
        figure = self.figure

        axes = figure.axes[0]

        print('n_steps', self.n_steps)
        for n in range(self.n_steps):

            axes.clear()

            phi_new = phi_old

            # use scipy-functionality to get the iterated value of phi_new
            # If the trial value calculated with phi_trial = phi_old
            # is smaller then target value get_lack_of_fit has no sign change
            # for phi_trial = phi_old and phi_trial = 0. which is a requirement
            # for the function call 'brentq'. In this case the old value
            # for phi_trial is used and tloop moves on one step
            try:
                # The method brentq has optional arguments such as
                #   'xtol'    - absolut error (default value = 1.0e-12)
                #   'rtol'    - relative error (not supported at the time)
                #   'maxiter' - maximum numbers of iterations used
                #
                # Here xtol is used to specify the allowed RELATIVE error!
                # therefore the relative lack of fit is returned in
                # method 'get_lack_of_fit'
                _xtol = 1.0e-6
                phi_new = brentq(self.get_lack_of_fit, 0., phi_old, xtol=_xtol)
                # @todo: check if 'brenth' gives better fitting results; faster?
#                phi_new = brenth( self.get_lack_of_fit, 0., phi_old )
                print('(#) n = ', n)
            except ValueError:

                if self.log:
                    lof_0 = self.get_lack_of_fit(0.)
                    lof_phi_old = self.get_lack_of_fit(phi_old)
                    print('No sign change between get_lack_of_fit(phi_old) = ', lof_phi_old, ' and ')
                    print('get_lack_of_fit(0.) = ', lof_0)
                    print('Use old value for phi_trial. phi_old = ', phi_old)
                else:
                    print('(!) n = ', n)

            # current time corresponds to the current strain applied
            #
            current_time = self.tloop.t_n

            # replace old 'phi_value' with iterated value:
            #
            phi_old = phi_new

            # get mats_state_array:
#            mats_state_array = copy(self.tloop.tstepper.sctx.mats_state_array)

            # update phi_data:
            #
            x = hstack([ self.fitted_phi_fn.xdata[:], current_time + self.step_size  ])
            y = hstack([ self.fitted_phi_fn.ydata[:], phi_new             ])

            axes.plot(x, y, color='blue', linewidth=2)
            self.data_changed = True

            self.fitted_phi_fn.set(xdata=x, ydata=y)
            self.fitted_phi_fn.data_changed = True

            # run one step with the iterated value for phi in order to
            # update the state array and to move forward one step:
            if self.log:
                print('\n')
                print('### run_one_step ###')
                print('### step', n   , '###')
                print('### current time:', current_time)

            if self.rec_trial_steps:
                # add entries of the iterations ('i') in the current step ('n')
                # (yields a list of lists)
                #
                self.phi_trial_list_n.append(self.phi_trial_list_i)
                self.sig_trial_list_n.append(self.sig_trial_list_i)
                # delete the entries of the iterations ('i') in the last step ('n')
                # and fill it with the iterations of the next step ('n+1')
                #
                self.phi_trial_list_i = []
                self.sig_trial_list_i = []

            self.run_one_step()
#            print '(g%)' %(n)

        self.fitted_phi_fn.changed = True
        mats_key = self.dim.mats_eval.__class__.__name__
        ctt_key = str(self.composite_tensile_test.key)
        if self.store_fitted_phi_fn:
            print("stored 'fitted_phi_fn' in CCSUnitCell with material model %s and calibration test %s" % (mats_key, ctt_key))
            print('ctt_key + self.param_key', ctt_key + self.param_key)
            self.composite_cross_section.set_param(mats_key, ctt_key + self.param_key,
#            self.composite_cross_section.set_param(mats_key, ctt_key,
                                                   copy(self.fitted_phi_fn))
            # save 'sig_eps_arr' in directory "/simdb/simdata/mats_calib_damage_fn"
            simdata_dir = os.path.join(simdb.simdata_dir, 'mats_calib_damage_fn')
            if os.path.isdir(simdata_dir) == False:
                os.makedirs(simdata_dir)
            ctt_key = str(self.composite_tensile_test.key)
            filename = os.path.join(simdata_dir, 'eps-phi-arr_' + ctt_key + self.param_key + '.csv')

            xdata, ydata = self.fitted_phi_fn.xdata[:, None], self.fitted_phi_fn.ydata[:, None]
            eps_phi_arr = np.hstack([xdata, ydata])
            np.savetxt(filename, eps_phi_arr, delimiter=';')
            print('eps-phi-data saved to file %s' % (filename))

    format_ticks = Bool(False)
    def plot_trial_steps(self):
        '''Plot target (sig-eps-curve of the tensile test) and trial curves
        and corresponding phi function together with trail steps from the iteration process.
        NOTE: the global variable 'rec_trial_steps' must be set to 'True' in order to store the iteration values
              within the global variables 'phi_trial_list_n' and 'sig_trial_list_n'
        n - index of the time steps to be considered
        i - index of the iteration steps performed in order to fit the target curve
        '''
        #-------------------------------------------------------------------
        # configure the style of the font to be used for labels and ticks
        #-------------------------------------------------------------------
        #
        from matplotlib.font_manager import FontProperties
        font = FontProperties()
#        font.serif         : Times, Palatino, New Century Schoolbook, Bookman, Computer Modern Roman
#        font.sans-serif    : Helvetica, Avant Garde, Computer Modern Sans serif
#        font.cursive       : Zapf Chancery
#        font.monospace     : Courier, Computer Modern Typewriter
        font.set_name('Script MT')
        # name = ['Times New Roman', 'Helvetica', 'Script MT'] #?
        font.set_family('serif')
        # family = ['serif', 'sans-serif', 'cursive', 'fantasy', 'monospace']
        font.set_style('normal')
        # style  = ['normal', 'italic', 'oblique']
        font.set_size('small')
        # size  = ['xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large', '11']
        font.set_variant('normal')
        # variant= ['normal', 'small-caps']
        font.set_weight('medium')
        # weight = ['light', 'normal', 'medium', 'semibold', 'bold', 'heavy', 'black']

        #-------------------------------------------------------------------

        p.figure(facecolor='white', dpi=600, figsize=(8, 6))  # white background

        # time list corresponding to the specified numbers of steps and step size
        #
        step_list = [n * self.step_size for n in range(self.n_steps + 1)]

        # get list of lists containing the trial values of 'sig_app' and 'phi_trail'
        # the lists are defined as global variables of 'MATSCalibDamageFn' and are filled
        # within the iteration process when the method 'get_lack_of_fit" is called
        #
        phi_trial_list_n = [[1.]] + self.phi_trial_list_n
        sig_trial_list_n = [[0.]] + self.sig_trial_list_n

        xrange = 10.  # plotting range for strain [mm/m]
        yrange = 15.  # plotting range for stress [MPa]

        for n in range(self.n_steps):
            for i in range(len(phi_trial_list_n[n + 1])):
                x = np.array([step_list[n], step_list[n + 1]])
                eps = 1000. * x  # plot strains in permil on the x-axis
                #--------------------------------------
                # sig-eps trial
                #--------------------------------------
                # plot the numerically calculated sig-eps-curve (tensile test)
                # (with trial steps)
                #
                sig_trail = np.array([sig_trial_list_n[n][-1], sig_trial_list_n[n + 1][i]])
                p.subplot(222)
                p.plot(eps, sig_trail, color='k', linewidth=1)
                p.xlabel(r'strain $\varepsilon$ [1E-3]', fontproperties=font)
                p.ylabel('stress $\sigma$ [MPa]', fontproperties=font)
                if self.format_ticks:
                    # format ticks for plot
                    p.axis([0, xrange, 0., yrange], fontproperties=font)
                    locs, labels = p.xticks()
                    p.xticks(locs, ["%.0f" % x for x in locs], fontproperties=font)
                    locs, labels = p.yticks()
                    p.yticks(locs, ["%.0f" % x for x in locs], fontproperties=font)

                #--------------------------------------
                # phi_trail
                #--------------------------------------
                # plot the fitted phi-function
                # (with trial steps)
                #
                p.subplot(224)
                phi_trail = np.array([phi_trial_list_n[n][-1], phi_trial_list_n[n + 1][i]])
                p.plot(eps, phi_trail, color='k', linewidth=1)
                p.xlabel(r'strain $\varepsilon$ [1E-3]', fontproperties=font)
                p.ylabel('integrity $\phi$ [-]', fontproperties=font)
                if self.format_ticks:
                    # format ticks for plot
                    p.yticks([0, 0.2, 0.4, 0.6, 0.8, 1.0])
                    p.axis([0, xrange, 0., 1.])
                    locs, labels = p.xticks()
                    p.xticks(locs, ["%.0f" % x for x in locs], fontproperties=font)
                    locs, labels = p.yticks()
                    p.yticks(locs, ["%.1f" % x for x in locs], fontproperties=font)

        #--------------------------------------
        # sig-eps target
        #--------------------------------------
        # plot the sig-eps-target curve (tensile test)
        #
        p.subplot(221)
        eps = 1000. * self.mfn_line_array_target.xdata[:-1]
        sig_target = self.mfn_line_array_target.ydata[:-1]
        p.plot(eps, sig_target, color='black', linewidth=1)
        p.xlabel(r'strain $\varepsilon$ [1E-3]', fontproperties=font)
        p.ylabel('stress $\sigma$ [MPa]', fontproperties=font)
        if self.format_ticks:
            # format ticks for plot
            p.axis([0, xrange, 0., yrange])
            locs, labels = p.xticks()
            p.xticks(locs, ["%.0f" % x for x in locs], fontproperties=font)
            locs, labels = p.yticks()
            p.yticks(locs, ["%.0f" % x for x in locs], fontproperties=font)

        #--------------------------------------
        # phi_trail (final)
        #--------------------------------------
        # plot the corresponding fitted phi-function
        # (without trial steps)
        #
        p.subplot(223)
        eps = 1000. * self.fitted_phi_fn.xdata[:-1]
        phi_fn = self.fitted_phi_fn.ydata[:-1]
        p.plot(eps, phi_fn, color='black', linewidth=1)
        p.xlabel(r'strain $\varepsilon$ [1E-3]', fontproperties=font)
        p.ylabel('integrity $\phi$ [-]', fontproperties=font)
        if self.format_ticks:
            # format ticks for plot
            p.yticks([0, 0.2, 0.4, 0.6, 0.8, 1.0])
            p.axis([0, xrange, 0., 1.])
            locs, labels = p.xticks()
            p.xticks(locs, ["%.0f" % x for x in locs], fontproperties=font)
            locs, labels = p.yticks()
            p.yticks(locs, ["%.1f" % x for x in locs], fontproperties=font)

        # save figure with calibration process in directory "/simdb/simdata/lcc_table/output_images/save_fig_to_file.png"
        simdata_dir = os.path.join(simdb.simdata_dir, 'mats_calib_damage_fn')
        if os.path.isdir(simdata_dir) == False:
            os.makedirs(simdata_dir)

        ctt_key = str(self.composite_tensile_test.key)
        filename = os.path.join(simdata_dir, ctt_key + self.param_key + '.pdf')
        p.savefig(filename)
        print('plot_trail_steps.png saved to file %s' % (filename))
        filename = os.path.join(simdata_dir, ctt_key + self.param_key + '.png')
        p.savefig(filename, dpi=600)
        print('plot_trail_steps.png saved to file %s' % (filename))

        p.show()

    #-----------------------------------------------------------------------------------------
    # User interaction
    #-----------------------------------------------------------------------------------------
    toolbar = ToolBar(
                  Action(name="Run Calibration",
                         tooltip='Run damage function calibration for the current parameters',
                         image=ImageResource('kt-start'),
                         action="run_calibration"),
                  image_size=(22, 22),
                  show_tool_names=False,
                  show_divider=True,
                  name='calibration_toolbar')

    traits_view = View(HSplit(
                            Item('ex_run@',
                                    show_label=False),
                            VSplit(
                               Item('dim@',
                                    id='mats_calib_damage_fn.run.split',
                                    dock='tab',
                                    resizable=True,
                                    label='experiment run',
                                    show_label=False),
                                    id='mats_calib_damage_fn.mode_plot_data.vsplit',
                                    dock='tab',
                                ),
                            VSplit(
                                Group(
                                      Item('figure', editor=MPLFigureEditor(),
                                         resizable=True, show_label=False),
                                    id='mats_calib_damage_fn.plot_sheet',
                                    label='fitted damage function',
                                    dock='tab',
                                    ),
                                    id='mats_calib_damage_fn.plot.vsplit',
                                    dock='tab',
                                   ),
                                    id='mats_calib_damage_fn.hsplit',
                                    dock='tab',
                                ),
#                        menubar = self.default_menubar(),
                        resizable=True,
                        toolbar=toolbar,
                        handler=MATSCalibDamageFnController(),
                        title='Simvisage: damage function calibration',
                        id='mats_calib_damage_fn',
                        dock='tab',
                        buttons=[ OKButton, CancelButton ],
                        height=0.8,
                        width=0.8)


def run():
    #--------------------------------------------------------------------------------
    # Example using the mats2d_explore
    #--------------------------------------------------------------------------------
    from ibvpy.mats.mats2D.mats2D_explore import MATS2DExplore
    from ibvpy.mats.mats2D.mats2D_rtrace_cylinder import MATS2DRTraceCylinder

    from ibvpy.mats.mats2D.mats2D_cmdm.mats2D_cmdm_rtrace_Gf_mic import \
        MATS2DMicroplaneDamageTraceGfmic, \
        MATS2DMicroplaneDamageTraceEtmic, MATS2DMicroplaneDamageTraceUtmic

    from ibvpy.mats.mats2D.mats2D_cmdm.mats2D_cmdm_rtrace_Gf_mac import \
        MATS2DMicroplaneDamageTraceGfmac, \
        MATS2DMicroplaneDamageTraceEtmac, MATS2DMicroplaneDamageTraceUtmac

    from ibvpy.mats.mats2D.mats2D_cmdm.mats2D_cmdm import \
        MATS2DMicroplaneDamage, MATS1DMicroplaneDamage, \
        PhiFnGeneral, PhiFnStrainHardening

    from ibvpy.api import RTraceGraph, RTraceArraySnapshot

    from bmcs_beam.mxn.mfn import MFnLineArray
    from numpy import array, hstack

    from ibvpy.mats.mats2D.mats2D_explorer_bcond import BCDofProportional
    from os.path import join

    ec = {
          # overload the default configuration
          'bcond_list'  : [ BCDofProportional(max_strain=1.0, alpha_rad=0.0) ],
          'rtrace_list' : [
               RTraceGraph(name='stress - strain',
                           var_x='eps_app', idx_x=0,
                           var_y='sig_app', idx_y=0,
                           record_on='iteration'),
                        ],
          }

    mats_eval = MATS2DMicroplaneDamage(
                                    n_mp=30,
    # mats_eval = MATS1DMicroplaneDamage(
                                    elastic_debug=False,
                                    stress_state='plane_stress',
                                    symmetrization='sum-type',
                                    model_version='compliance',
                                    phi_fn=PhiFnGeneral,
                                    )

#    print 'normals', mats_eval._MPN
#    print 'weights', mats_eval._MPW

    fitter = MATSCalibDamageFn(KMAX=300,
                               tolerance=5e-4,  # 0.01,
                               RESETMAX=0,
                               dim=MATS2DExplore(
                                                 mats_eval=mats_eval,
                                                 explorer_config=ec,
                                                 ),
                               store_fitted_phi_fn=True,
                               log=False
                               )

    #-------------------------------------------
    # run fitter for entire available test data:
    #-------------------------------------------

    calibrate_all = False

    if calibrate_all:
        from matresdev.db.exdb.ex_run_table import \
            ExRunClassExt
#         from matresdev.db.exdb.ex_composite_tensile_test import \
#             ExCompositeTensileTest
#         ex = ExRunClassExt(klass=ExCompositeTensileTest)
#         for ex_run in ex.ex_run_list:
#             if ex_run.ready_for_calibration:
#                 print 'FITTING', ex_run.ex_type.key
#                 # 'E_c' of each test is different, therefore 'mats_eval'
#                 # needs to be defined for each test separately.
#                 #
#                 E_c = ex_run.ex_type.E_c
#                 nu = ex_run.ex_type.ccs.concrete_mixture_ref.nu
#
#                 # run calibration
#                 #
#                 fitter.ex_run = ex_run
#                 fitter.dim.mats_eval.E = E_c
#                 fitter.dim.mats_eval.nu = nu
#                 fitter.init()
#                 fitter.fit_response()

    else:

        test_file = join(simdb.exdata_dir,
                              'tensile_tests',
                              'dog_bone',
#                              'buttstrap_clamping',

                              '2010-02-09_TT-10g-3cm-a-TR_TRC11',
#                              'TT11-10a-average.DAT' )
                              'TT-10g-3cm-a-TR-average.DAT')

                               #-----------------------------------
                               # tests for 'BT-3PT-12c-6cm-TU_ZiE'
                               #-----------------------------------
                               # 'ZiE-S1': test series no. 1 (age = 11d)
                               #
#                                 '2011-05-23_TT-12c-6cm-0-TU_ZiE',
#                                 'TT-12c-6cm-0-TU-V2.DAT')

                                # 'ZiE-S2': test series no. 2 (age = 9d)
                                #
#                                 '2011-06-10_TT-12c-6cm-0-TU_ZiE',
#                                 'TT-12c-6cm-0-TU-V2.DAT')

                                #-----------------------------------
                                # tests for 'BT-4PT-12c-6cm-TU_SH4'
                                # tests for 'ST-12c-6cm-TU' (fresh)
                                #-----------------------------------
                                # @todo: add missing front strain information from Aramis3d testing
                                #
#                               '2012-04-12_TT-12c-6cm-0-TU_SH4-Aramis3d',
#                               'TT-12c-6cm-0-TU-SH4-V2.DAT')

#                                '2012-02-14_TT-12c-6cm-0-TU_SH2',
#                                'TT-12c-6cm-0-TU-SH2-V2.DAT')

#                                '2012-02-14_TT-12c-6cm-0-TU_SH2',
#                                'TT-12c-6cm-0-TU-SH2F-V3.DAT')

                                # used for suco(!)
#                                '2012-02-14_TT-12c-6cm-0-TU_SH2',
#                                'TT-12c-6cm-0-TU-SH2-V1.DAT')

                                #-----------------------------------
                                # tests for 'BT-3PT-6c-2cm-TU_bs'
                                #-----------------------------------
                                # barrelshell
                                #
#                                 # TT-bs1
#                                 '2013-05-17_TT-6c-2cm-0-TU_bs1',
#                                 'TT-6c-2cm-0-TU-V3_bs1.DAT')
#                                 # TT-bs2
#                                 '2013-05-21-TT-6c-2cm-0-TU_bs2',
#                                 'TT-6c-2cm-0-TU-V1_bs2.DAT')
#                                 # TT-bs3
#                                 '2013-06-12_TT-6c-2cm-0-TU_bs3',
#                                 'TT-6c-2cm-0-TU-V1_bs3.DAT')
#                                 # TTb-bs4-Aramis3d
#                                  '2013-07-09_TTb-6c-2cm-0-TU_bs4-Aramis3d',
#                                  'TTb-6c-2cm-0-TU-V2_bs4.DAT')

                                #-----------------------------------
                                # tests for 'TT-6c-2cm-90-TU'
                                #-----------------------------------
                                #
#                                '2013-05-22_TTb-6c-2cm-90-TU-V3_bs1',
#                                'TTb-6c-2cm-90-TU-V3_bs1.DAT')

#                               '2013-05-17_TT-6c-2cm-0-TU_bs1',
#                               'TT-6c-2cm-90-TU-V3_bs1.DAT')

#        test_file = join(simdb.exdata_dir,
#                               'tensile_tests',
#                               'buttstrap_clamping',
#                               '2013-07-18_TTb-6c-2cm-0-TU_bs5',
#                               'TTb-6c-2cm-0-TU-V1_bs5.DAT')
# #                               'TTb-6c-2cm-0-TU-V3_bs5.DAT')
#
#        test_file = join(simdb.exdata_dir,
#                               'tensile_tests',
#                               'buttstrap_clamping',
#                               '2013-07-09_TTb-6c-2cm-0-TU_bs4-Aramis3d',
#                               'TTb-6c-2cm-0-TU-V2_bs4.DAT')

                                #-----------------------------------
                                # tests for 'TT-6g-2cm-0-TU' (ARG-1200-TU)
                                #-----------------------------------
                                #
        # test series no.1
        #
#        test_file = join(simdb.exdata_dir,
#                               'tensile_tests',
#                               'dog_bone',
#                               '2012-12-10_TT-6g-2cm-0-TU_bs',
#                               'TT-6g-2cm-0-V2.DAT')
        # test series no.3
        #
#        test_file = join(simdb.exdata_dir,
#                               'tensile_tests',
#                               'buttstrap_clamping',
#                               '2013-07-09_TTb-6g-2cm-0-TU_bs4-Aramis3d',
#                               'TTb-6g-2cm-0-TU-V1_bs4.DAT')

        # test series NxM_1
        #
#        test_file = join(simdb.exdata_dir,
#                          'tensile_tests',
#                          'buttstrap_clamping',
#                          '2014-04-30_TTb-6c-2cm-0-TU_NxM1',
#                          'TTb-6c-2cm-0-TU-V16_NxM1.DAT')

        #------------------------------------------------------------------
        # set 'ex_run' of 'fitter' to selected calibration test
        #------------------------------------------------------------------
        #
        ex_run = ExRun(data_file=test_file)
        fitter.ex_run = ex_run

        #------------------------------------------------------------------
        # specify the parameters used within the calibration
        #------------------------------------------------------------------
        #
        # get the composite E-modulus and Poisson's ratio as stored
        # in the experiment data base for the specified age of the tensile test
        #
        E_c = ex_run.ex_type.E_c
        print('E_c', E_c)

#        # use the value as graphically determined from the tensile test (= initial stiffness for tension)
#        E_c = 28000.

        # age, Em(age), and nu of the slab test or bending test determines the
        # calibration parameters. Those are used for calibration and are store in the 'param_key'
        # appendet to the calibration-test-key
        #
        age = 28

        # E-modulus of the concrete matrix at the age of testing
        # NOTE: value is more relevant as compression behavior is determined by it in the bending tests and slab tests;
        # behavior in the tensile zone is defined by calibrated 'phi_fn' with the predefined 'E_m'
#        E_m = ex_run.ex_type.ccs.get_E_m_time(age)
        E_c = ex_run.ex_type.ccs.get_E_c_time(age)

        # use average E-modul from 0- and 90-degree direction for fitter in both directions
        # this yields the correct tensile behavior and returns the best average compressive behavior
        #
#        E_c = 22313.4

        # alternatively use maximum E-modul from 90-direction also for 0-degree direction for fitting
        # this yields the correct tensile behavior also in the linear elastic regime for both directions corresponding to the
        # tensile test behavior (note that the compressive E-Modulus in this case is overestimated in 0-degree direction; minor influence
        # assumed as behavior is governed by inelastic tensile behavior and anisotropic redistrirbution;
        #
#        E_c = 29940.2
#        E_c = 29100.
#        E_c = 22390.4
#        E_c = 18709.5
        E_c = 28700.

        # smallest value for matrix E-modulus obtained from cylinder tests (d=150mm)
#        E_m = 18709.5

        # set 'nu'
        # @todo: check values stored in 'mat_db'
        #
        nu = 0.20
        ex_run.ex_type.ccs.concrete_mixture_ref.nu = nu

        n_steps = 200
        fitter.n_steps = n_steps

        fitter.format_ticks = True

        fitter.ex_run.ex_type.age = age
        print('age = %g used for calibration' % age)
        fitter.ex_run = ex_run

#        print 'E_m(age) = %g used for calibration' % E_m
#        fitter.dim.mats_eval.E = E_m

        print('E_c(age) = %g used for calibration' % E_c)
        fitter.dim.mats_eval.E = E_c

        print('nu = %g used for calibration' % nu)
        fitter.dim.mats_eval.nu = nu

        print('n_steps = %g used for calibration' % n_steps)

        max_eps = fitter.max_eps
        print('max_eps = %g used for calibration' % max_eps)

        #------------------------------------------------------------------
        # set 'param_key' of 'fitter' to store calibration params in the name
        #------------------------------------------------------------------
        #
#        param_key = '_age%g_Em%g_nu%g_nsteps%g' % (age, E_m, nu, n_steps)
#        param_key = '_age%g_Ec%g_nu%g_nsteps%g__smoothed' % (age, E_c, nu, n_steps, max_eps)
        param_key = '_age%g_Ec%g_nu%g_nsteps%g_smoothed' % (age, E_c, nu, n_steps)

        fitter.param_key = param_key
        print('param_key = %s used in calibration name' % param_key)

        #------------------------------------------------------------------
        # run fitting procedure
        #------------------------------------------------------------------
        #
        fitter.init()
        fitter.fit_response()
        fitter.plot_trial_steps()

    return

    #---------------------------
    # basic testing of fitter methods:
    #---------------------------

    # set to True for basic testing of the methods:
    basic_tests = False

    if basic_tests:
        fitter.run_through()
        #    fitter.tloop.rtrace_mngr.rtrace_bound_list[0].configure_traits()
        fitter.tloop.rtrace_mngr.rtrace_bound_list[0].redraw()
        last_strain_run_through = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.xdata[:]
        last_stress_run_through = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.ydata[:]
        print('last strain (run-through) value' , last_strain_run_through)
        print('last stress (run-through) value' , last_stress_run_through)

        fitter.tloop.reset()
        fitter.run_step_by_step()
        # fitter.tloop.rtrace_mngr.rtrace_bound_list[0].configure_traits()
        fitter.tloop.rtrace_mngr.rtrace_bound_list[0].redraw()
        last_strain_step_by_step = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.xdata[:]
        last_stress_step_by_step = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.ydata[:]
        print('last stress (step-by-step) value', last_stress_step_by_step)

        fitter.run_trial_step()
        fitter.run_trial_step()
        fitter.tloop.rtrace_mngr.rtrace_bound_list[0].redraw()
        strain_after_trial_steps = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.xdata[:]
        stress_after_trial_steps = fitter.tloop.rtrace_mngr.rtrace_bound_list[0].trace.ydata[:]
        print('stress after trial', stress_after_trial_steps)

        fitter.init()
        # fitter.mats2D_eval.configure_traits()
        lof = fitter.get_lack_of_fit(1.0)
        print('1', lof)
        lof = fitter.get_lack_of_fit(0.9)
        print('2', lof)

        # fitter.tloop.rtrace_mngr.configure_traits()
        fitter.run_trial_step()

    else:
        from ibvpy.plugins.ibvpy_app import IBVPyApp
        ibvpy_app = IBVPyApp(ibv_resource=fitter)
        ibvpy_app.main()

if __name__ == '__main__':
    run()




'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, List

import numpy as np

from .reinf_law_base import \
    ReinfLawBase

class ReinfLawLinear(ReinfLawBase):
    '''Effective crack bridge Law with linear elastic response.'''

    eps_u = Float(0.01, enter_set=True, auto_set=False, input=True)
    E_tex = Float(80000, enter_set=True, auto_set=False, input=True)
    u0 = List([ 0.01, 80000. ], enter_set=True, auto_set=False)

    sig_tex_u = Property(depends_on='+input')
    @cached_property
    def _get_sig_tex_u(self):
        return self.E_tex * self.eps_u

    cnames = ['eps_u', 'E_tex']

    eps_arr = Property(depends_on='+input')
    @cached_property
    def _get_eps_arr(self):
        return np.array([ 0., self.eps_u])

    sig_arr = Property(depends_on='+input')
    @cached_property
    def _get_sig_arr(self):
        # with limit for eps_tex
        #
        return self.E_tex * self.eps_arr

ReinfLawBase.db.constants['linear-default'] = ReinfLawLinear()



from .material_type_base import \
    MaterialTypeBase

from .material_type_matrix_mixture import \
    MTMatrixMixture

from .material_type_reinf_bar import \
    MTReinfBar

from .material_type_reinf_fabric import \
    MTReinfFabric



# Import Standard Libraries
import logging
import scipy as np
import matplotlib.pyplot as plt

# Import Local Libraries
import Util_ACI as ACI
import Util_EC2 as EC2
from RC import ReinforcedBeam


#===========================================================================
#   Prestressed Beam
#===========================================================================

class PrestressedBeam(ReinforcedBeam):
	""" Abstract Reinforced Beam Class
	
	Static Members:
		gamma_c = 1.5
		gamma_S = 1.15
		gamma_P = 1.1
		niter = 20

	Instance Members:
	
	Public Methods:
		[alpha, beta, la, eta] = design_parameters(self)
		[fcd, fyd, fp01d, fpd] = design_properties(self)
	
	Static Methods:
		plotArrow(self, ax, x, y, dx, dy, color)
		plotStress(self, ax, fcd, h_top, h_bot) 
	"""
	gamma_P = 1.1
	hf = bf = bw = yb = e_P = 0
	d_P = d_P0 = Ap = fpk = Ep = Pm = 0

	def __init__(self):
		ReinforcedBeam.__init__(self)
		self.fig = plt.figure(figsize=(9, 3))
		self.sigma_p = self.Pm/self.Ap
		self.f = self.d_P - self.d_P0

	def design_parameters(self):
		[alpha, beta] = EC2.alpha_beta(self.fck, self.units)
		[la, eta] = EC2.lambda_eta(self.fck, self.units)
		logging.debug("    a = {:3.2f}, b = {:3.2f}".format(alpha, beta))
		logging.debug("    lambda = {:3.2f}, eta = {:3.2f}".format(la, eta))
		return [alpha, beta, la, eta]

	def design_properties(self):
		fcd = self.fck/self.gamma_c
		fyd = self.fyk/self.gamma_S
		fp01d = 0.9*self.fpk/self.gamma_P
		fpd = self.fpk/self.gamma_P
		logging.debug("    fcd = {:6.2f}, fyd = {:6.2f}".format(fcd, fyd))
		logging.debug("    fp01d = {:6.2f}, fpd = {:6.2f}".format(fp01d, fpd))
		return [fcd, fyd, fp01d, fpd]

	def EC2_design_moment(self):
		[alpha, beta, la, eta] = self.design_parameters()
		[fcd, fyd, fp01d, fpd] = self.design_properties()
		ecu3 = EC2.ultimate_strain(self.fck, self.units)
		epy = fp01d/self.Ep
		epu = 0.035
		ey = fyd/self.Es
		ax = self.fig.add_subplot(132)
		ax.plot([0, 0], [0, self.H], linewidth=2, color='k')

		# Assume fp = 0.95*fpd and fs = fyd
		fp = 0.95*fpd
		fs = fyd
		logging.debug("    Assume fp = 0.95 fpd = {:6.2f}".format(fp))

		Xu = (self.Ap*(fp-self.sigma_p)+self.Pm)/(alpha*self.bf*fcd)

		for iter in range(self.niter):

			if Xu < self.hf:
				Xu = (self.Ap*(fp-self.sigma_p)+self.Pm)/(alpha*self.bf*fcd)

			elif Xu > self.hf:
				X = (self.Ap*fp-eta*fcd*self.hf *
                                    (self.bf-self.bw))/(eta*fcd*self.bw)
				Xu = X/la

			logging.debug(
				"Iteration {:1.0f}: \n    Xu = {:6.2f}".format(iter, Xu))

			# Is the assumption correct?
			Delta_ep = (self.d_P-Xu)*ecu3/Xu
			ep = self.sigma_p/self.Ep + Delta_ep
			fp_new = fp01d + (ep-epy)*(fpd-fp01d)/(epu-epy)
			logging.debug("    ep = {:6.5f}, fp = {:6.2f}".format(ep, fp_new))

			es = (self.d_S-Xu)*ecu3/Xu
			fs_new = fyd + (es-ey)*self.k

			# Approximation Error

			if abs((fp_new-fp)/fp_new) < 0.001 and abs((fs_new-fs)/fs_new) < 0.001:
				break

			# New assumption f = f_new
			fp = fp_new
			fs = fs_new

		# Moment Capacity
		if Xu < self.hf:
			# _________
			#   |  |     ||
			#   |  |     ||   b*Xu
			#   |  |_    ||<------ Nc
			#   |    \_  ||    |
			#   |      \_||    y1
			#   |        ||    |
			#   dp ----- ||<------ Pm　＠　neutral axis
			#   |    |   ||    y2
			#   |   e_P  ||<------ Ns
			#   |    |   ||
			#  --------- ||<------Delta Pm
			#            ||

			Nc = alpha*self.bf*Xu*fcd
			y1 = self.d_P - self.e_P - beta*Xu
			logging.info("    Nc = {:6.2f}, y1 = {:6.2f}".format(Nc, y1))

			Ns = self.As*fs
			y2 = self.d_S - y1 - beta*Xu
			logging.info("    Ns = {:6.2f}, y2 = {:6.2f}".format(Ns, y2))

			Delta_P = self.Ap*fp - self.Pm
			logging.info(
				"    Delta_P = {:6.2f}, e_P = {:6.2f}".format(Delta_P, self.e_P))

			MRd = Nc*y1 + Ns*y2 + Delta_P*self.e_P

		elif Xu > self.hf:
			# _________
			#   |  |     ||
			#   |  |     ||<----------- Nc1
			#   |  |_____||<---- Nc2 |
			#   |  |     ||   |      |
			#   |  |_____||   y2     y1
			#   |        ||   |      |
			#   dp ----  ||<------------ Pm　＠　neutral axis
			#   |    |   ||    y3
			#   |   e_P  ||<------ Ns
			#   |    |   ||
			#  --------- ||<------Delta Pm
			#            ||

			Nc1 = (self.bf-self.bw)*self.hf*fcd
			y1 = self.d_P - self.e_P - self.hf/2
			self.plotStress(ax, fcd, self.H, self.H - self.hf)
			self.plotArrow(ax, Nc1/10**5, self.yb+y1, -Nc1/10**5, 0, 'b')
			logging.info("    Nc1 = {:6.2f}, y1 = {:6.2f}".format(Nc1, y1))

			Nc2 = self.bw*X*fcd
			y2 = self.d_P - self.e_P - X/2
			self.plotStress(ax, fcd, self.H, self.H - Xu)
			self.plotArrow(ax, Nc2/10**5, self.yb+y2, -Nc2/10**5, 0, 'b')
			logging.info("    Nc2 = {:6.2f}, y2 = {:6.2f}".format(Nc2, y2))

			Ns = self.As*fs
			y3 = self.d_P - y1 - self.hf/2
			logging.info("    Ns = {:6.2f}, y3 = {:6.2f}".format(Ns, y3))

			Delta_P = self.Ap*fp - self.Pm
			self.plotArrow(ax, 0, self.H - self.d_P, Delta_P/10**5, 0, 'r')
			logging.info(
				"    Delta_P = {:6.2f}, e_P = {:6.2f}".format(Delta_P, self.e_P))

			MRd = Nc1*y1 + Nc2*y2 + Ns*y3 + Delta_P*self.e_P
			logging.info("    MRd = {:5.2f}".format(MRd))
			plt.xlim((-1.2*fcd, 2*fcd))
		return MRd

	def rotation_capacity(self):
		pass

#===========================================================================
#   Rectangular Beam
#===========================================================================


class RectangularBeam(PrestressedBeam):
	h = 1000
	d_P = 800
	e_P = 300
	b = 800
	Ap = 4300
	fck = 45
	fpk = 1860

	Ep = 195000
	Pm = 4800000
	units = "MPa"

	def __init__(self):

		self.sigma_p = self.Pm/self.Ap
		self.Ac = self.b*self.h
		self.I = self.b*self.h**3/12
		self.W = self.b*self.h**2/6

	def EC2_design_moment(self):
		[alpha, beta, la, eta] = self.design_parameters()
		[fcd, fyd, fp01d, fpd] = self.design_properties()
		ecu3 = EC2.ultimate_strain(self.fck, self.units)
		epy = fp01d/self.Ep
		epu = 0.035

		# Assume fp = 0.95*fpd
		fp = 0.95*fpd
		logging.debug("    Assume fp = 0.95fpd = {:6.2f}".format(fp))
		for iter in range(self.niter):

			# Find Xu using horizontal equilibrium
			Xu = (self.Ap*(fp-self.sigma_p)+self.Pm)/(alpha*self.b*fcd)
			logging.debug(
				"Iteration {:1.0f}: \n    Xu = {:6.2f}".format(iter, Xu))

			# Is the assumption correct?
			Delta_ep = (self.d_P-Xu)*ecu3/Xu
			ep = self.sigma_p/self.Ep + Delta_ep
			fp_new = fp01d + (ep-epy)*(fpd-fp01d)/(epu-epy)
			logging.debug("    ep = {:6.5f}, fp = {:6.2f}".format(ep, fp_new))

			# Approximation Error
			if abs((fp_new-fp)/fp_new) < 0.01:
				break
			# New assumption fp = fp_new
			fp = fp_new

		# Moment Capacity
		Nc = alpha*self.b*Xu*fcd
		y1 = self.d_P - self.e_P - beta*Xu
		Delta_P = self.Ap*fp_new - self.Pm
		MRd = Nc*y1 + Delta_P*self.e_P
		logging.info("    MRd = {:5.2f}".format(MRd))
		return MRd



import tkinter as tk

counter = 0


def counter_label(label):
  def count():
    global counter
    counter += 1
    label.config(text=str(counter))
    label.after(1000, count)
  count()


root = tk.Tk()
root.title("Counting Seconds")
label = tk.Label(root, fg="green")
label.pack()
counter_label(label)
button = tk.Button(root, text='Stop', width=25, command=root.destroy)
button.pack()
root.mainloop()



'''
Created on 22. 4. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

from bmcs_beam.mxn.material_types import \
    MTReinfFabric

from bmcs_beam.mxn.reinf_laws import \
    ReinfLawFBM, ReinfLawLinear

import numpy as np

import pickle

from bmcs_beam.mxn.utils import \
    get_outfile

if MTReinfFabric.db.get('fabric-test', None):
    del MTReinfFabric.db['fabric-test']
MTReinfFabric.db['fabric-test'] = MTReinfFabric(A_roving=0.461,
                                                s_0=0.0083,
                                                s_90=0.02,
                                                mtrl_laws=
                                                {'fbm':
                                                ReinfLawFBM(sig_tex_u=1000.,
                                                            eps_u=0.014,
                                                            m=0.5),
                                                'linear':
                                                ReinfLawLinear(eps_u=0.014,
                                                                E_tex=65000.),
                                                }
                                                )

def test_reinf_fabric_states():
    '''Test the moment and normal force calculated for a cross section
    with changing reinforcement law.
    '''
    cp = CrossSection(reinf=[RLCTexUniform(n_layers=6, material='default_fabric', material_law='cubic')],
                         matrix_cs=MatrixCrossSection(geo=MCSGeoRect(width=0.1, height=0.05),
                                                      n_cj=20, material_law='constant',
                                                      material='default_mixture'),
                         eps_lo=0.014,
                         eps_up=-0.0033,
                         )

    MTReinfFabric.db['default_fabric'].mtrl_laws['cubic'].set(sig_tex_u=1216., eps_u=0.016, var_a=-5e+6)
    MTReinfFabric.db['default_fabric'].set(s_0=0.0083, A_roving=0.461)
    MTReinfFabric.db['default_fabric'].mtrl_laws['fbm'].set(sig_tex_u=1216., eps_u=0.014, m=0.5)

    assert np.allclose([cp.M, cp.N], [1.186453793394266, -30.616264285738165])
    cp.reinf_components_with_state[0].material_law = 'fbm'
    assert np.allclose([cp.M, cp.N], [1.1864949677288525, -24.578077501696075])
    cp.reinf_components_with_state[0].material = 'fabric-test'

    object_file = get_outfile(folder_name='.mxn',
                             file_name='test08_reinf_fabric.pkl')
    pickle.dump(cp, open(object_file, 'wb'), 1)
    assert np.allclose([cp.M, cp.N], [1.1604352105840721, -28.960062926594549])
    MTReinfFabric.db['fabric-test'].A_roving = 0.6
    assert np.allclose([cp.M, cp.N], [1.1968125127766132, -22.843167549587157])
    MTReinfFabric.db['fabric-test'].s_0 = 0.004
    assert np.allclose([cp.M, cp.N], [1.3823867162441215, 8.3614001029031542])
    MTReinfFabric.db['fabric-test'].mtrl_laws['fbm'].sig_tex_u = 1200.
    assert np.allclose([cp.M, cp.N], [1.4509064221398169, 19.883086620745736])
    cp.reinf_components_with_state[0].material_law = 'linear'
    assert np.allclose([cp.M, cp.N], [1.3150963500309509, -18.425889629166878])

    loaded_cp = pickle.load(open(object_file, 'rb'))
    assert np.allclose([loaded_cp.M, loaded_cp.N], [1.4509064221398169, 19.883086620745736])
    MTReinfFabric.db['fabric-test'].mtrl_laws['fbm'].sig_tex_u = 1000.
    assert np.allclose([loaded_cp.M, loaded_cp.N], [1.3823867162441215, 8.3614001029031542])
    loaded_cp.reinf_components_with_state[0].material_law = 'linear'
    assert np.allclose([loaded_cp.M, loaded_cp.N], [1.3150963500309509, -18.425889629166878])

if __name__ == '__main__':
    test_reinf_fabric_states()



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Created on May 25, 2009 by Rostislav Chudoba
#
#-------------------------------------------------------------------------

# Enthought library imports
import csv
import os
import sys
import wx

from chaco.api import \
    add_default_axes, \
    add_default_grids, OverlayPlotContainer, \
    PlotGraphicsContext, \
    ArrayDataSource, \
    DataRange1D, LinePlot, LinearMapper
from chaco.tools.api import \
    PanTool, \
    BroadcasterTool, ZoomTool
from enable.api import \
    Window
from pyface.api import FileDialog, OK
from traits.api import \
    Range,  \
    Any,  Instance, HasPrivateTraits
from traitsui.basic_editor_factory import \
    BasicEditorFactory
from traitsui.menu import \
    Action, ToolBar
from traitsui.wx.editor import \
    Editor

from etsproxy.pyface.api \
    import ImageResource
from .mfn_plot_adapter import \
    MFnPlotAdapter
import numpy as np


# from kiva.traits.kiva_font_trait import \
#     KivaFont
#from util.resource import find_resource
# from etsproxy.traits.ui.editor_factory import \
#    EditorFactory
USE_DATA_UPDATE = 1
WILDCARD = "Saved plots (*.png)|*.png|(*.jpg)|*.jpg|"\
           "All files (*.*)|*.*"

# Range for the height and width for the plot widget.
PlotSize = Range(50, 1000, 180)


class _MFnChacoEditor (Editor):
    """ Traits UI editor for displaying trait values in a MFnLine.
    """

    # TableEditorToolbar associated with the editor:
    toolbar = Any
    # The Traits UI associated with the function editor toolbar:
    #  toolbar_ui = Instance( UI )

    # adjustable parameters
    adapter = Instance(MFnPlotAdapter)

    lplot = Instance(LinePlot)

    #---------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------

    plot_container = Instance(OverlayPlotContainer)

    def _plot_container_default(self):
        container = OverlayPlotContainer(padding=50,
                                         fill_padding=False,
                                         bgcolor="white",
                                         use_backbuffer=True)

        return container

    #---------------------------------------------------------------------
    #  Finishes initializing the editor by creating the underlying toolkit
    #  widget:
    #---------------------------------------------------------------------
        """ Finishes initializing the editor by creating the 
            underlying toolkit widget.
        """

    def init(self, parent):
        factory = self.factory
        self.adapter = factory.adapter
        self.control = self._create_canvas(parent)
        # Register the update listener
        #
        self.value.on_trait_change(self.update_editor, 'data_changed')

    def update_editor(self):
        c = self.plot_container
#        c.padding = 30
        c.remove(*c.components)
        self._refresh_container()

    def _create_canvas(self, parent):
        '''Create canvas for chaco plots
        '''
        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
#        self.panel = panel

        container_panel = Window(panel, component=self.plot_container)

        a = self.adapter
        if a.max_size:
            panel.SetMaxSize(a.max_size)
        if a.min_size:
            panel.SetMinSize(a.min_size)

        print('size', panel.GetSize())

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(wx.StaticLine(parent, -1, style=wx.LI_HORIZONTAL), 0,
                  wx.EXPAND | wx.BOTTOM, 5)
        sizer.Add(self._create_toolbar(panel), 0, wx.EXPAND)
        sizer.Add(container_panel.control, 1, wx.EXPAND)

        sizer.Add(wx.StaticLine(parent, -1, style=wx.LI_HORIZONTAL), 0,
                  wx.EXPAND | wx.BOTTOM, 5)

        panel.SetSizer(sizer)

        if a.padding:
            for side, pad in list(a.padding.items()):
                if side == 'right' or side == 'top':
                    padd = int((1 - pad) * 200)
                    setattr(self.plot_container, 'padding_' + side, padd)
                else:
                    padd = int(pad * 200)
                    setattr(self.plot_container, 'padding_' + side, padd)

        return panel

    def _refresh_container(self):
        ''' rebuild the container for the current data
        '''
        broadcaster = BroadcasterTool()

        mfn_line = self.value
     #   print self.panel.GetSize()
        adapter = self.adapter
        if adapter.var_x != '':
            # Get the x-label text from the object's trait var_x
            label_x = getattr(self.object, adapter.var_x)
        else:
            # Get the x-label from the adapter
            label_x = adapter.label_x

        if adapter.var_y != '':
            label_y = getattr(self.object, adapter.var_y)
        else:
            label_y = adapter.label_y

        index = ArrayDataSource(mfn_line.xdata)
        y = ArrayDataSource(mfn_line.ydata, sort_order="none")

        index_range = DataRange1D()
        index_range.add(index)
        index_mapper = LinearMapper(range=index_range)

        value_range = DataRange1D(low_setting=0.0)
        value_range.add(y)
        value_mapper = LinearMapper(range=value_range)

        styles_m = list(adapter.line_style.values())
        line_style = styles_m[0]
        line_color = adapter.line_color[0]

        line_plot = self.lplot = LinePlot(index=index, value=y,
                                          index_mapper=index_mapper,
                                          value_mapper=value_mapper,
                                          color=line_color,
                                          width=25,
                                          edge_color='blue',
                                          linestyle=line_style,
                                          border_visible=False)

        add_default_grids(line_plot)
        add_default_axes(line_plot, vtitle="Y", htitle="X")

        self.plot_container.add(line_plot)
        line_plot.tools.append(PanTool(line_plot))
        line_plot.overlays.append(ZoomTool(line_plot))

#        pan = PanTool(line_plot)
#        zoom = SimpleZoom(line_plot, tool_mode="box", always_on=False)
#        broadcaster.tools.append(pan)
#        broadcaster.tools.append(zoom)
#
#        # Add the traits inspector tool to the container
#        #
#        self.plot_container.tools.append(TraitsTool( self.plot_container ))

    #--------------------------------------------------------------------------
    #  Creates the table editing tool bar:
    #--------------------------------------------------------------------------

    def _create_toolbar(self, parent):
        """ Creates the table editing toolbar.
        """
        factory = self.factory

        panel = wx.Panel(parent, -1, style=wx.CLIP_CHILDREN)
        toolbar = MFnChacoEditorToolbar(parent=parent, editor=self)
        tb_sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel.SetSizer(tb_sizer)

        self.toolbar = toolbar
        tb_sizer.Add(toolbar.control, 0)
        tb_sizer.Add((1, 1), 1, wx.EXPAND)

        return panel

    #--------------------------------------------------------------------------
    #  Handles the user requesting that columns not be sorted:
    #--------------------------------------------------------------------------
    def on_savedata(self):
        """ Handles the user requesting that 
            the data of the function is to be saved.
        """
        import os
        dlg = FileDialog(parent=self.control,
                         title='Export function data',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard='*.csv',
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving data to", path, "...")
            try:

                #                factory  = self.factory
                #                plotitem = factory.plotitem
                #                x_values = getattr(self.object, plotitem.index)
                #                y_values = getattr(self.object, plotitem.value)
                x_values = self.value.xdata
                y_values = self.value.ydata
                np.savetxt(path, np.vstack((x_values, y_values)).transpose())
            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return

    def on_savefig(self):
        """ Handles the user requesting that the image of the 
            function is to be saved.
        """
        import os
        dlg = FileDialog(parent=self.control,
                         title='Save as image',
                         default_directory=os.getcwd(),
                         default_filename="", wildcard=WILDCARD,
                         action='save as')
        if dlg.open() == OK:
            path = dlg.path

            print("Saving plot to", path, "...")
            try:

                """ Now we create a canvas of the appropriate size and 
                    ask it to render our component.  
                   (If we wanted to display this plot in a window, we
                    would not need to create the graphics context ourselves; 
                    it would be created for us by the window.)
                """
#                self._plot.bounds = [500,300]
#                self._plot.padding = 50
               # plot_gc = PlotGraphicsContext(self._plot.outer_bounds)
               # plot_gc.render_component(self._plot)

                #self._plot_container.outer_bounds = list((800,600))
#                plot_gc = PlotGraphicsContext((400,300), dpi=72.0)
#                plot_gc.render_component(self._plot_container)

                self.lplot.bounds = [500, 300]
                self.lplot.padding = 50

                win_size = self.lplot.outer_bounds
                plot_gc = PlotGraphicsContext(win_size)

                # Have the plot component into it
                plot_gc.render_component(self.lplot)

                # Finally, we tell the graphics context
                # to save itself to disk as an image.
                plot_gc.save(path)

            except:
                print("Error saving!")
                raise
            print("Plot saved.")
        return

#------------------------------------------------------------------------------
#  'MFnChacoEditorToolbar' class:
#------------------------------------------------------------------------------


class MFnChacoEditorToolbar (HasPrivateTraits):
    """ Toolbar displayed in table editors.
    """
    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Do not sort columns:
    save_data = Instance(Action,
                         {'name':    'Save as data',
                          'tooltip': 'Save the function values',
                          'action':  'on_savedata',
                          'enabled': True,
                          'image':   ImageResource('add')})

    # Move current object up one row:
    save_fig = Instance(Action,
                        {'name':    'Save as fig',
                         'tooltip': 'Save as figure',
                         'action':  'on_savefig',
                         'enabled': True,
                         'image':   ImageResource('save')})

    # The table editor that this is the toolbar for:
    editor = Instance(_MFnChacoEditor)

    # The toolbar control:
    control = Any

    #-------------------------------------------------------------------------
    #  Initializes the toolbar for a specified window:
    #-------------------------------------------------------------------------

    def __init__(self, parent=None, **traits):
        super(MFnChacoEditorToolbar, self).__init__(**traits)
        factory = self.editor.factory

        actions = [self.save_data, self.save_fig]
        toolbar = ToolBar(image_size=(16, 16),
                          show_tool_names=False,
                          show_divider=False,
                          *actions)
        self.control = toolbar.create_tool_bar(parent, self)
        self.control.SetBackgroundColour(parent.GetBackgroundColour())

        # fixme: Why do we have to explictly set the size of the toolbar?
        #        Is there some method that needs to be called to do the
        #        layout?
        self.control.SetSize(wx.Size(23 * len(actions), 16))

    #-------------------------------------------------------------------------
    #  PyFace/Traits menu/toolbar controller interface:
    #-------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    #  Adds a menu item to the menu bar being constructed:
    #-------------------------------------------------------------------------
    def add_to_menu(self, menu_item):
        """ Adds a menu item to the menu bar being constructed.
        """
        pass
    #-------------------------------------------------------------------------
    #  Adds a tool bar item to the tool bar being constructed:
    #-------------------------------------------------------------------------

    def add_to_toolbar(self, toolbar_item):
        """ Adds a toolbar item to the too bar being constructed.
        """
        pass
    #-------------------------------------------------------------------------
    #  Returns whether the menu action should be defined in the user interface:
    #-------------------------------------------------------------------------

    def can_add_to_menu(self, action):
        """ Returns whether the action should be defined in the user interface.
        """
        return True
    #-------------------------------------------------------------------------
    #  Returns whether the toolbar action should be defined in the user
    #  interface:
    #-------------------------------------------------------------------------

    def can_add_to_toolbar(self, action):
        """ Returns whether the toolbar action should be defined in the user
            interface.
        """
        return True
    #-------------------------------------------------------------------------
    #  Performs the action described by a specified Action object:
    #-------------------------------------------------------------------------

    def perform(self, action, action_event=None):
        """ Performs the action described by a specified Action object.
        """
        getattr(self.editor, action.action)()


class MFnChacoEditor(BasicEditorFactory):
    """ Editor factory for plot editors.
    """

    klass = _MFnChacoEditor

    adapter = Instance(MFnPlotAdapter)

    def _adapter_default(self):
        return MFnPlotAdapter()



'''
Created on 19. 3. 2014

@author: Vancikv

An example of a parameter study script - an alternative
to the tree view interface in creating a pstudy.
'''

from bmcs_beam.mxn import \
    CrossSection, MxNDiagram

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

from matplotlib.figure import \
    Figure

from matplotlib.backends.backend_agg import \
    FigureCanvasAgg

from bmcs_beam.mxn.utils import \
    get_outfile

ge = MCSGeoRect(height=0.06, width=0.14)
mcs = MatrixCrossSection(geo=ge, n_cj=20,
                         material='default_mixture',
                         material_law='constant')
uni_layers = RLCTexUniform(n_layers=12,
                           material='default_fabric',
                           material_law='fbm')
'''Geometry, matrix and reinforcement
'''

uni_layers.material_law_.sig_tex_u = 1200.13
'''
N = 103 kN
sig_tex_u = N / A_tex_tot = 103000 / (12 * 16 * 0.447) = 1200.13 N/mm2
'''

cs = CrossSection(matrix_cs=mcs, reinf=[uni_layers])
'''Cross section object
'''

mxn = MxNDiagram(cs=cs, n_eps=20)
'''MxN diagram object
'''

fig = Figure(figsize=(10, 7), dpi=80, facecolor='white')
canvas = FigureCanvasAgg(fig)
ax = fig.add_subplot(1, 1, 1)
'''Create something to plot into
'''

mxn.plot_custom(ax, color='black', linestyle='-',
                linewidth=2.0, label='fbm')

uni_layers.material_law = 'bilinear'
mxn.plot_custom(ax, color='black', linestyle='--',
                linewidth=2.0, label='bilinear')

uni_layers.material_law = 'linear'
mxn.plot_custom(ax, color='black', linestyle='-.',
                linewidth=2.0, label='linear')

uni_layers.material_law = 'cubic'
mxn.plot_custom(ax, color='black', linestyle=':',
                linewidth=2.0, label='cubic')
'''Parameter study - Plot the mxn diagram for different
reinforcement laws. Equivalently when using the tree view,
the plot_custom function is used by the PStudyElement
object which contains the plotting specs for its MxNDiagram.
'''

ax.legend()
ax.set_xlabel('M[kNm]')
ax.set_ylabel('N[kN]')

canvas.print_figure(get_outfile(folder_name='.mxn',
                                file_name='ex11.png'))



'''
Created on Apr 9, 2010

Scipt generates a csv-file based on a ASC-file containing the test data of a
tensile test with four gauge displacements[mm].
The displacement gauge with name "W20_hre" needs to be multiplied by factor 2
due to a false machine setup.
@author: alexander
'''

from numpy import \
    array, shape, hstack, savetxt, loadtxt

from os.path import \
    join

from matresdev.db.simdb import \
    SimDB


# name of the tensile test / ASC-file with gauge displacements
#
TT_name = 'TT-12c-6cm-TU-SH1-V1'
TT_name = 'TT-12c-6cm-TU-SH1-V2'
TT_name = 'TT-12c-6cm-TU-SH1-V3'
TT_name = 'TT-12c-6cm-TU-SH1F-V1'
TT_name = 'TT-12c-6cm-TU-SH1F-V2'
TT_name = 'TT-12c-6cm-TU-SH1F-V3'


# file path in simdb
#
simdb = SimDB()
file_path = join( simdb.exdata_dir, 'tensile_tests', '2012-01-09_TT-12c-6cm-TU-SH1' )
file_name_ASC = join( file_path, TT_name + '.ASC' )

print('read input data from file ' + file_name_ASC + '\n')

# open file
#
file = open( file_name_ASC, 'r' )

data_arr = loadtxt( file_name_ASC,
                    delimiter = ";" )

# prepare outputarr multiplying the value of the displacement gauge 
# "W20_hre" by factor 2.
WA_hre = ( data_arr[:, -1] )[:, None]

output_arr = hstack( ( data_arr[:, 0:-1], 2.0 * WA_hre ) )
print('output_arr', output_arr)

file_name_csv = join( file_path, TT_name + '.csv' )

print('save output data to file ' + file_name_csv + '\n')

savetxt( file_name_csv, output_arr, delimiter = ';' )




# -------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Aug 3, 2009 by: rch
#
# How to introduce ExperimentType class - as a class corresponding to SimModel
#
# - this class gathers the information about the inputs and outputs
#   of the object.
#
# - Each directory contains the `ex_type.cls' file giving the name of hte
#   ExperimentType subclass defining the inputs and outputs of the experiment
#   and further derived data needed for processing of experiments.
#
# - The ExperimentDB has a root directory and starts by scanning the
#   sub-directories
#   for the `ex_type.cls' files. It verifies that the ExTypes are defined
#   and known
#   classes. Then, the mapping between a class and between the directories
#   is established. Typically, there is a data pool in the home directory
#   or a network file system accessible.
#
# - The result of the scanning procedure
#   is list of data directories available for
#   each experiment type. Typically, data from a single treatment are grouped
#   in the single directory, but this does not necessarily need to be the case.
#   Therefore, the grouping is done independently on the data based on
#   the values of the input factors.
#
#   Tasks: define the classes
#     - ETCompositeTensileTest
#     - ETPlateTest
#     - ETPullOutTest,
#     - ETYarnTensileTest
#
#   They inherit from class ExType specifying the inputs and outputs.
#   They also specify
#   the default association of tracers to the x- and y- axis (plot templates).
#   Further, grouping
#   of equivalent values can be provided. The values of the inputs are stored
#   in the directory in the ExType.db file using the pickle format.
#
#   The ExTypeView class communicates with the particular ExType class.
#   It should be able
#   to accommodate several curves within the plotting window.
#   It should be possible to
#   select multiple runs to be plotted.
#

# - Database of material components is stored in a separate directory tree.
#   The identification
#   of the material components is provided using a universal key
#   for a material component The component must be declared either as a matrix
#   or reinforcement. It must be globally accessible from within
#   the ExpTools and SimTools.
#

import os
from os.path import join
import pickle
from traits.api import \
    HasTraits, \
    on_trait_change, File, Instance, Trait, \
    Property, cached_property, \
    Bool, Event, implements
from traitsui.api import \
    View, Item, \
    FileEditor
from traitsui.menu import \
    OKButton, CancelButton

import apptools.persistence.state_pickler as spickle
from .ex_type import \
    ExType
from .i_ex_run import \
    IExRun
from .i_ex_type import \
    IExType
from matresdev.db.simdb import simdb
from util.find_class import _find_class


data_file_editor = FileEditor(filter=['*.DAT'])

# which pickle format to use
#
pickle_modes = {'pickle': dict(load=pickle.load,
                               dump=pickle.dump,
                               ext='.pickle'),
                'spickle': dict(load=spickle.load_state,
                                dump=spickle.dump,
                                ext='.spickle')}


class ExRun(HasTraits):

    '''Read the data from the DAT file containing the measured data.

    The data is described in semicolon-separated
    csv file providing the information about
    data parameters.

    '''
    implements(IExRun)

    data_file = File
    '''File containing the association between the factor combinations
    and data files having the data.
    '''

    pickle = Trait('pickle', pickle_modes)
    '''Pickle object as a property in order to be able to switch between
    '''

    ex_type_file_name = Property(depends_on='data_file')
    '''Derived  path to the file specifying the type of the experiment,
    the ex_type.cls file is stored in the same directory.
    '''
    @cached_property
    def _get_ex_type_file_name(self):
        dir_path = os.path.dirname(self.data_file)
        file_name = os.path.join(dir_path, 'ex_type.cls')
        return file_name

    pickle_file_name = Property(depends_on='data_file')
    '''Derived path to the pickle file storing the input data and derived output
    data associated with the experiment run.
    '''
    @cached_property
    def _get_pickle_file_name(self):
        dir_path = os.path.dirname(self.data_file)
        file_name = os.path.basename(self.data_file)
        file_split = file_name.split('.')
        file_name = os.path.join(dir_path,
                                 file_split[0] + self.pickle_['ext'])
        return file_name

    ex_type = Instance(IExType)
    '''Instance of the specialized experiment type with the particular
    inputs and data derived outputs.
    '''

    def _ex_type_default(self):
        return ExType()

    def __init__(self, data_file, **kw):
        '''Initialization: the ex_run is defined by the
        data_file. The additional data - inputs and derived outputs
        are stored in the data_file.pickle. If this file exists,
        the exrun is constructed from this data file.
        '''
        super(ExRun, self).__init__(**kw)
        self.data_file = data_file
        read_ok = False

        if os.path.exists(self.pickle_file_name):
            print('PICKLE FILE EXISTS %s' % self.pickle_file_name)
            file_ = open(self.pickle_file_name, 'r')
            try:
                self.ex_type = self.pickle_['load'](file_)
                self.unsaved = False
                read_ok = True
            except EOFError:
                read_ok = False

            file_.close()
            self.ex_type.data_file = self.data_file

            # In case that the code for processing data
            # has changed since the last dump - run
            # the processing anew. This can be skipped
            # if the code is finished.
            #
            # @todo - the refreshing of the process data
            # should be possible for all instances at once
            # this would avoid the - on-click-based refreshing
            # of the data performed here.
            #
            self.ex_type.process_source_data()
            self.ex_type.derived_data_available = True
            print('*** derived data available ***')

        if not read_ok and os.path.exists(self.ex_type_file_name):

            print('PICKLE FILE DOES NOT EXIST')

            f = open(self.ex_type_file_name, 'r')
            ex_type_klass = f.read().split('\n')[0]  # use trim here
            f.close()
            theClass = _find_class(ex_type_klass)
            if theClass is None:
                raise TypeError('class %s not found for file %s' % (ex_type_klass, self.ex_type_file_name))
            self.ex_type = theClass(data_file=self.data_file)
            self.unsaved = True
            read_ok = True

            self.ex_type.process_source_data()

        if not read_ok:
            raise ValueError('Cannot instantiate ExType using %s' % data_file)

    def save_pickle(self):
        '''Store the current state of the ex_run.
        '''
        file_ = open(self.pickle_file_name, 'w')
        self.pickle_['dump'](self.ex_type, file_)
        file_.close()
        self.unsaved = False

    # Event to keep track of changes in the ex_type instance.
    # It is defined in order to inform the views about a change
    # in some input variable.
    #
    change_event = Event

    # Boolean variable set to true if the object has been changed.
    # keep track of changes in the ex_type instance. This variable
    # indicates that the run is unsaved. The change may be later
    # canceled or confirmed.
    #
    unsaved = Bool(transient=True)

    @on_trait_change('ex_type.input_change')
    def _set_changed_state(self):
        print('*** received input change event ***''')
        self.change_event = True
        self.unsaved = True

    # Boolean property indicating whether the run is suitable and prepared
    # for calibration of models.
    ready_for_calibration = Property(Bool)

    def _get_ready_for_calibration(self):

        # it must have a pickle data containing the values
        # of the test parameters
        #
        return (os.path.exists(self.ex_type_file_name) and
                self.ex_type.ready_for_calibration)

    # --------------------------------------------------------------------------
    # View specification
    # --------------------------------------------------------------------------
    view_traits = View(
        Item('ex_type@', show_label=False,
             resizable=True,
             label='experiment type'
             ),
        resizable=True,
        scrollable=True,
        # title = 'Data reader',
        id='simexdb.exrun',
        dock='tab',
        buttons=[OKButton, CancelButton],
        height=0.8,
        width=0.8)


if __name__ == '__main__':
    test_file = join(simdb.exdata_dir, 'tensile_tests', 'TT-9u',
                     'TT06-9u-V1.DAT')
#     test_file = join(simdb.exdata_dir, 'plate_tests', 'PT-10a',
#                               'PT10-10a.DAT')
    exrun = ExRun(test_file)
    exrun.configure_traits(view='view_traits')
    print('processed data', exrun.ex_type.data_array.shape)
    print('ex_type file name', exrun.ex_type_file_name)
    print('ex_type', exrun.ex_type)
    print('pickle_file_name', exrun.pickle_file_name)

    # exrun.save_pickle()
    # exrun = ExRun( '/home/rch/sim_data/sim_exdb/ex_composite_tensile_test/TT08-7a-V2.DAT' )
    # exrun.configure_traits()



'''
Created on Apr 1, 2010

@author: alexander
'''
from traits.api import \
    HasTraits, Directory, List, Int, Float, Any, \
    on_trait_change, File, Constant, Instance, Trait, \
    Array, Str, Property, cached_property, WeakRef, \
    Dict, Button, Bool, Enum, Event, implements, \
    DelegatesTo, Callable

from traitsui.api import \
    View, Item, DirectoryEditor, TabularEditor, HSplit, VGroup, \
    TableEditor, EnumEditor, Handler, FileEditor, VSplit, Group, \
    InstanceEditor, HGroup, Spring

# # overload the 'get_label' method from 'Item' to display units in the label
from util.traits.ui.item import \
    Item

from numpy import \
    log

from matresdev.db.simdb.simdb_class import \
    SimDBClass, SimDBClassExt

class ConcreteMixture(SimDBClass):
    '''Describes the properties of the concrete matrix
    '''

    # E-modulus of the concrete after 28d
    E_m28 = Float(unit='MPa', simdb=True, input=False, auto_set=False, enter_set=False)

    # E-modulus of the concrete after 28d
    nu = Float(unit='-', simdb=True, input=False, auto_set=False, enter_set=False)

    # developement of the E-modulus depending on the age at the time of testing:
    get_E_m_time = Callable

    # view:
    traits_view = View(
                      Item('key'  , style='readonly'),
                      Item('E_m28', style='readonly', format_str="%.0f"),
                      Item('nu'   , style='readonly', format_str="%.2f"),
                      resizable=True,
                      scrollable=True
                      )

# Setup the database class extension
#
ConcreteMixture.db = SimDBClassExt(
            klass=ConcreteMixture,
            constants={
                # NOTE: log = natural logarithm  ("ln")

                'PZ-0708-1' : ConcreteMixture(
                                           E_m28=33036.,
                                           get_E_m_time=lambda t: 4665. * log(t + 0.024) + 17487.,
                                           nu=0.25
                                           ),
                'FIL-10-09' : ConcreteMixture(
                                           E_m28=28700.,
                                           # function for the evolution derived based on only
                                           # three values: Em0 = 0, Em7 = 23600, Em28 = 28700
                                           get_E_m_time=lambda t: 3682. * log(t + 0.012) + 16429.,
                                           nu=0.25
                                           ),
                'FIL-Standard-SF' : ConcreteMixture(
                                           E_m28=31000.,
                                           # function for the evolution unknown
                                           get_E_m_time=lambda t: 31000.,
                                           nu=0.2
                                           ),
                'barrelshell' : ConcreteMixture(
#                                           E_m28 = 22721., #
                                           E_m28=19800.,  # based on cylinder tests
                                           get_E_m_time=lambda t: 19800.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.25
                                           ),
                'shotcrete-4mm' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           # @todo: function for the evolution derived based on only
                                           # three values: Em0 = 0, Em7 = 23600, Em28 = 28700
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),

                'sto-100' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           # function for the evolution derived based on only
                                           # three values: Em0 = 0, Em7 = 23600, Em28 = 28700
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),

                'flowstone' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           # function for the evolution derived based on only
                                           # three values: Em0 = 0, Em7 = 23600, Em28 = 28700
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),

                'Pagel_TF10' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           # function for the evolution derived based on only
                                           # three values: Em0 = 0, Em7 = 23600, Em28 = 28700
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),
                'HPC_TU_WIEN' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),
                'UHPC_TU_WIEN' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),
                'HPC_SF1.5_TU_WIEN' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),
                'UHPC_SF2.0_TU_WIEN' : ConcreteMixture(
                                           E_m28=30000.,  # @todo: approximation only
                                           get_E_m_time=lambda t: 30000.,  # @todo: specify time function for evolution of E-modulus; so far value for 28d is used;
                                           nu=0.2  # @todo: approximation only
                                           ),
                                           }
            )

if __name__ == '__main__':
    ConcreteMixture.db.configure_traits()



'''
Created on 6. 8. 2014

@author: Vancikv

In this example a parametric study of an MxN diagram of a steel_design
reinforced cross section with different reinforcement ratios is
constructed - i.e. a dimensioning diagram that can be
compared with literature, e.g. Schneider Bautabellen.

Specifications:

Geometry:  rectangle 40x40cm
Concrete:  C30/37
steel_design:     B500B
d1/h = 0.1

Reinforcement ratios: 0, 0.5, 1.0, 1.5, 2.0

As_tot = omega_tot * (b * h) / (f_yd / f_cd)
As1 = As_tot / 2    (Reinforcement on one side for certain omega)

For omega_tot = 0.5:
As1 = 1.0 * (0.4 * 0.4) / (435 / 17) / 2 = 15.640e-4 m^2
'''
from bmcs_beam.mxn import \
    CrossSection, MxNDiagram

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCBar

from bmcs_beam.mxn.matrix_laws import \
    MatrixLawBlock

from bmcs_beam.mxn.reinf_laws import \
    ReinfLawSteel

from bmcs_beam.mxn.material_types import \
    MTMatrixMixture, MTReinfBar

from bmcs_beam.mxn.view import \
    MxNTreeView

from bmcs_beam.mxn.use_cases import \
    UseCaseContainer, UCParametricStudy

if MTMatrixMixture.db.get('ex12_C30-37', None):
    del MTMatrixMixture.db['ex12_C30-37']
MTMatrixMixture.db['ex12_C30-37'] = MTMatrixMixture(f_ck=9.,
                                                eps_c_u=0.0035,
                                                mtrl_laws={'constant':
                                                            MatrixLawBlock(f_ck=9.,
                                                                           eps_c_u=0.0035,
                                                                           E_c=33e+3),
                                         })

if MTReinfBar.db.get('ex12_bar', None):
    del MTReinfBar.db['ex12_bar']
MTReinfBar.db['ex12_bar'] = MTReinfBar(area=0.001564,
                                          mtrl_laws={'steel_design':
                                                    ReinfLawSteel(f_yk=500 / 1.15)
                                         })
'''Defining materials
'''

ge = MCSGeoRect(height=0.4, width=0.4)
mcs = MatrixCrossSection(geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
'''Geometry, matrix
'''

cs00 = CrossSection(reinf=[],
                    matrix_cs=MatrixCrossSection(
                         geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
                    )
cs05 = CrossSection(reinf=[RLCBar(x=0.2, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.2, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           ],
                    matrix_cs=MatrixCrossSection(
                         geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
                    )
cs10 = CrossSection(reinf=[RLCBar(x=0.1, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.1, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.3, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.3, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           ],
                    matrix_cs=MatrixCrossSection(
                         geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
                    )
cs15 = CrossSection(reinf=[RLCBar(x=0.1, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.1, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.2, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.2, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.3, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.3, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           ],
                    matrix_cs=MatrixCrossSection(
                         geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
                    )
cs20 = CrossSection(reinf=[RLCBar(x=0.05, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.05, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.15, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.15, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.25, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.25, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.35, z=0.36,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           RLCBar(x=0.35, z=0.04,
                                  material='ex12_bar',
                                  material_law='steel_design'),
                           ],
                    matrix_cs=MatrixCrossSection(
                         geo=MCSGeoRect(height=0.4, width=0.4),
                         n_cj=20,
                         material='ex12_C30-37',
                         material_law='constant')
                    )
'''Defining cross sections with different reinforcement ratios
'''

mxn_ps = UCParametricStudy()
mxn_ps.element_to_add = 'mxndiagram'
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'omega = 0.0'
mxn_ps.tree_node_list[-1].color = 'blue'
mxn_ps.tree_node_list[-1].content = MxNDiagram(cs=cs00, n_eps=40)
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'omega = 0.5'
mxn_ps.tree_node_list[-1].linestyle = 'dashed'
mxn_ps.tree_node_list[-1].content = MxNDiagram(cs=cs05, n_eps=40)
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'omega = 1.0'
mxn_ps.tree_node_list[-1].linestyle = 'dash_dot'
mxn_ps.tree_node_list[-1].content = MxNDiagram(cs=cs10, n_eps=40)
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'omega = 1.5'
mxn_ps.tree_node_list[-1].linestyle = 'dotted'
mxn_ps.tree_node_list[-1].content = MxNDiagram(cs=cs15, n_eps=40)
mxn_ps.add_element = True
mxn_ps.tree_node_list[-1].node_name = 'omega = 2.0'
mxn_ps.tree_node_list[-1].content = MxNDiagram(cs=cs20, n_eps=40)
'''Defining a parametric study object and feeding it MxN diagrams
with the designated cross sections
'''

ucc = UseCaseContainer()
ucc.tree_node_list.append(mxn_ps)

mxn_ps_view = MxNTreeView(root=ucc)
mxn_ps_view.configure_traits()



# Import Standard Libraries
import logging

#===========================================================================
#   Utilities
#===========================================================================


def convert_2_psi(fck, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return fck if units == "psi" else 145.038*fck


def convert_2_in(h, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return h if units == "psi" else h/25.4


def convert_2_in2(A, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return A if units == "psi" else A/(25.4**2)


def convert_2_pcf(rho, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    if units == "psi":
        return rho
    elif units == "MPa" and rho != 145:
        return 6.366*rho


def convert_2_MPa(fck, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return fck if units == "MPa" else fck/145.038


def convert_2_mm(h, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return h if units == "MPa" else 25.4*h


def convert_2_mm2(A, units):
    if units not in ["MPa", "psi"]:
        raise KeyError("Only psi or MPa")
    return A if units == "MPa" else 25.4**2*A

def compare_steel_area(As,A_min,A_max):
    if As > A_min:
        logging.info(
            "    As = {:6.2f} > A_min = {:6.2f}. Md > Mcr".format(As, A_min))
    else:
        logging.info(
            "    As = {:6.2f} < A_min = {:6.2f}. Md < Mcr".format(As, A_min))
    if As < A_max:
        logging.info(
            "    As = {:6.2f} < A_max = {:6.2f}. OK".format(As, A_max))
    else:
        logging.info(
            "    As = {:6.2f} > A_max = {:6.2f}. Not OK".format(As, A_max))



'''
Created on 26. 6. 2014

@author: Vancikv
'''

import weakref
import copy

from bmcs_beam.mxn.constitutive_law import \
    CLBase
from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode
from traits.api import \
    Property, cached_property, Dict, Str, \
    on_trait_change, List, Button

from bmcs_beam.mxn.matresdev.db.simdb import \
    SimDBClass


class MaterialTypeBase(MxNTreeNode, SimDBClass):

    mtrl_laws = Dict((Str, CLBase))

    #=========================================================================
    # Management of backward links
    #=========================================================================

    state_link_lst = List(transient=True)
    '''List of backward links to objects using the fabric
    '''

    def _state_link_lst_default(self):
        return []

    @on_trait_change('+geo_input,+law_input')
    def notify_change(self):
        for link in self.state_link_lst:
            if link():
                link().material_changed = True

    def add_link(self, link_to_add):
        '''Adding a backward link to the list - to be called
        from objects using the fabric
        '''
        if link_to_add not in self.state_link_lst:
            self.state_link_lst.append(weakref.ref(link_to_add))

    def del_link(self, link_to_del):
        '''Removing a backward link from the list - to be called
        from objects using the fabric
        '''
        self.state_link_lst[:] = [
            link for link in self.state_link_lst if link() != link_to_del]

    #=========================================================================
    # UI-related functionality
    #=========================================================================

    new_law = Button(label='Add new law')

    def _new_material_fired(self):
        pass

    del_law = Button(label='Delete law')

    def _del_material_fired(self):
        pass

    law_keys = Property

    def _get_law_keys(self):
        return list(self.mtrl_laws.keys())

    chosen_law = Str()

    def _chosen_law_default(self):
        return list(self.mtrl_laws.keys())[0]

    tree_node_list = Property(depends_on='mtrl_laws')

    @cached_property
    def _get_tree_node_list(self):
        return list(self.named_mtrl_laws.values())

    @on_trait_change('node_name')
    def update_key(self):
        if self.key != self.node_name and self.key != '':
            old_key = self.key
            self.key = self.node_name
            self.db[self.node_name] = self
            self.save()
            del self.db[old_key]



# Import Standard Libraries
import logging
import scipy as np
import matplotlib.pyplot as plt

# Import Local Libraries
import Util_ACI as ACI
import Util_EC2 as EC2
from PSC import PrestressedBeam
from TeeBeam import TeeBeam

#===========================================================================
#   BoxGirder
#===========================================================================


class BoxGirder(PrestressedBeam):
	Bt = 1500
	ht = 250

	Bb = 1200
	hb = 250
	
	bt = 1000
	bb = 800
	H = 1000
	
	d_S = 800
	d_P = 800
	d_P0 = 300
	As = 0
	Ap = 10080

	fck = 50
	fyk = 500
	fpk = 1860
	
	Es = 205000 
	k = 0.01
	Ep = 195000
	Pm = 11344030
	units = "MPa"

	#   |<--------------- Bt --------------->|
	#	______________________________________  ___
	#	|                       |            |   |
	#	|                       |            |   |   ___
	#	|                       ht           |   |    |
	#	|         ______________|___         |   |    |
	#   |<-bwt/2->|<------bt----|->|<-bwt/2->|   |    |
	#	|         |             |  |         |   |    |
	#   |         |            hw  |         |   H    |   ___
	#   |         |             |  |         |   |   yft   |
	#   |         |<------bb----|->|         |   |    |    |
	#	|<-bwb/2->|_____________|__|<-bwb/2->|   |    |   yfw
	#	|                       |            |   |    |    |   ___
	#	|                       |            |   |    |    |    |
	#	|                       hb           |   |    |    |   yfb
	#	|_______________________|____________|  _|_  _|_  _|_  _|_
	#
	#   |<--------------- Bb --------------->|

	def __init__(self):

		PrestressedBeam.__init__(self)

		# Slope of girder width
		self.slope = (self.Bt - self.Bb)/self.H		
		
		# Properties of top flange
		Bt2 = self.Bt - self.slope*self.ht
		self.hf = self.ht # Needed for EC2_design_moment
		self.bf = (self.Bt + Bt2)/2
		Aft = (self.Bt + Bt2)*self.ht/2
		Ift = self.momentInertia(self.ht, self.Bt, Bt2)
		yft = self.H - self.neutralAxis(self.ht, self.Bt, Bt2)
		logging.debug("    Aft = {:6.2f}, Ift = {:10.2f}".format(Aft, Ift))

		# Properties of bottom flange
		Bb2 = self.Bb + self.slope*self.hb
		Afb = (self.Bb + Bb2)*self.hb/2
		Ifb = self.momentInertia(self.hb, self.Bb, Bb2)
		yfb = self.neutralAxis(self.hb, self.Bb, Bb2)
		logging.debug("    Afb = {:6.2f}, Ifb = {:10.2f}".format(Afb, Ifb))

		# Properties of web
		hw = self.H - self.ht - self.hb
		self.bwt = Bt2 - self.bt
		self.bwb = Bb2 - self.bb
		self.bw = (self.bwt + self.bwb)/2 # Needed for EC2_design_moment
		Aw = (self.bwt + self.bwb)*hw/2
		Iw = self.momentInertia(hw, self.bwb, self.bwt)
		yw = self.hb + self.neutralAxis(hw, self.bwb, self.bwt)
		logging.debug("    Aw = {:6.2f}, Iw = {:10.2f}".format(Aw, Iw))

		# Properties of BoxGirder
		self.Ac = Aft + Aw + Afb
		self.yb = (Aft*yft + Aw*yw + Afb*yfb)/self.Ac
		self.yt = self.H - self.yb
		logging.debug("    yt = {:6.2f}, yb = {:}".format(self.yb, self.yt))

		self.e_P = self.d_P - self.yt
		logging.debug("    e_P = {:6.2f}".format(self.e_P))

		self.I = Ift + Iw + Ifb + Aft*(yft - self.yb)**2 \
			+ Aw*(yw - self.yb)**2 + Afb*(yfb - self.yb)**2
		logging.debug("    Ac = {:6.2f}, I = {:10.2f}".format(self.Ac, self.I))

		self.Wt = self.I/self.yt
		self.Wb = self.I/self.yb

		logging.debug("    Wt = {:6.2f}, Wb = {:6.2f}".format(self.Wt, self.Wb))

	def plotBoxGirder(self):
		X1 = self.Bt
		X3 = self.H*self.slope/2
		X2 = X3 + self.Bb

		Y0 = self.H

		x0 = self.ht*self.slope/2 + self.bwt/2
		x1 = x0 + self.bt
		x3 = (self.H - self.hb)*self.slope/2 + self.bwb/2
		x2 = x3 + self.bb

		y0 = self.H - self.ht
		y1 = self.hb

		ax = self.fig.add_subplot(131)
		X = [0, X1, X2, X3, 0]
		Y = [Y0, Y0, 0, 0, Y0]
		ax.plot(X, Y, linewidth=1, color='k')
		ax.fill(X, Y, color = 'lightblue')

		X = [x0, x1, x2, x3, x0]
		Y = [y0, y0, y1, y1, y0]
		ax.plot(X, Y, linewidth=1, color='k')
		ax.fill(X, Y, color = 'w')

		ax.plot([0, self.Bt], [self.yb, self.yb], linestyle='--', color='k')

		ax.plot(self.Bt/2, self.yb, marker='+', color='k')

		if self.Ap != 0:
			ax.plot(self.Bt/2, self.H-self.d_P, marker='o', color='r')
			ax.plot(self.Bt/2, self.H-self.d_P0, marker='o', color='r')

		if self.As != 0:
			ax.plot(self.Bt/2, self.H-self.d_S, marker='o', color='k')

		plt.show()

	def plotStressDist(self, Xu, *args):

		ax = self.fig.add_subplot(132)
		ax.plot([0, 0], [0, self.h], linewidth=2, color='k')

		# Moment Capacity
		if Xu < self.ht:
			Nc1 = args[0]
			print(Nc1)
		elif Xu > self.ht:

			[fcd, Nc1, y1, Nc2, y2, Ns, y3] = args

			h0 = self.h - self.ht
			self.__plotConcreteStress(ax, fcd, self.h, h0)

			h1 = self.h - Xu
			self.__plotConcreteStress(ax, fcd, self.h, h1)

			# self.__plotSteelStress(ax, Ns, y3)
			print(Ns)
			# y1 = [self.h, self.h, h1, h1, self.h]

			h1 = h1 + Xu/2

			plt.xlim((-1.2*fcd, 1.2*fcd))
			plt.show()


if __name__ == "__main__":
	logging.basicConfig(level=logging.DEBUG, format='%(message)s')
	beam = BoxGirder()
	beam.EC2_design_moment()
	beam.plotBoxGirder()
	# beam.plotStressDist()



'''
Created on Apr 9, 2010

@author: alexander
'''

from numpy import array, vstack, zeros

from os.path import \
    join

from matresdev.db.simdb.simdb import simdb

# The implementation works but is too time consuming.
# @todo: check for a faster and more simple solution!


def loadtxt_novalue(file_name):
    '''Return an data array similar to loadtxt. 
    "NOVALUE" entries are replaced by the value of the previous line.
    '''
    file = open(file_name, 'r')
    lines = file.readlines()
    n_columns = len(lines[0].split(';'))
    data_array = zeros(n_columns)
    n = 0
    for line in lines:
        line_split = lines[n].split(';')
        m = 0
        for value in line_split:
            if value == 'NOVALUE':
                print('---------------------------------------------------------------')
                print('NOVALUE entry in line', n, 'position', m, 'found')
                print('For faster processing replace values directly in the data file!')
                print('---------------------------------------------------------------')
                line_split[m] = lines[n - 1].split(';')[m]
            m += 1
        line_array = array(line_split, dtype=float)
        data_array = vstack([data_array, line_array])
        n += 1
    return data_array


if __name__ == '__main__':
    ex_path = join(
        simdb.exdata_dir, 'plate_tests', 'PT-10a', 'PT11-10a_original.ASC')
    data_array = loadtxt_novalue(ex_path)
    print('\n')
    print('data_array', data_array)



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, List

import numpy as np

from .reinf_law_base import \
    ReinfLawBase

class ReinfLawCubic(ReinfLawBase):
    '''Effective crack bridge Law using a cubic polynomial.'''

    sig_tex_u = Float(1216., enter_set=True, auto_set=False, input=True)
    eps_u = Float(0.016, enter_set=True, auto_set=False, input=True)
    var_a = Float(-5e+6, enter_set=True, auto_set=False, input=True)

    cnames = ['eps_u', 'var_a']

    u0 = List([ 0.016, -5000000. ])

    eps_arr = Property(depends_on='+input')
    @cached_property
    def _get_eps_arr(self):
        return np.linspace(0, self.eps_u, num=100.)

    sig_arr = Property(depends_on='+input')
    @cached_property
    def _get_sig_arr(self):
        # for horizontal tangent at eps_u
        sig_tex_u, var_a, eps_u = self.sig_tex_u, self.var_a, self.eps_u
        eps_arr = self.eps_arr
        var_b = -(sig_tex_u + 2. * var_a * eps_u ** 3.) / eps_u ** 2.
        var_c = -3. * var_a * eps_u ** 2. - 2. * var_b * eps_u
        sig_arr = var_a * eps_arr ** 3. + var_b * eps_arr ** 2. + var_c * eps_arr
        return sig_arr

ReinfLawBase.db.constants['cubic-default'] = ReinfLawCubic()



'''
Created on 1. 7. 2014

@author: Vancikv
'''
from traits.api import \
    List, Instance

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

from bmcs_beam.mxn.material_types import \
    MTMatrixMixture, MTReinfFabric, MTReinfBar

class UCDatabase(MxNTreeNode):
    node_name = 'Material database'

    tree_node_list = List(Instance(MxNTreeNode), transient=True)
    def _tree_node_list_default(self):
        return [MTMatrixMixture.db,
                      MTReinfFabric.db,
                      MTReinfBar.db]




from .mfn_polar import MFnPolar
import unittest

class TestSequenceFunctions(unittest.TestCase):
    
    def setUp(self):
        self.mp = MFnPolar( alpha = 0.7, delta_alpha = 0.25, delta_trans = 1. )

    def test_get_value(self):
        '''
        make sure that the values for corner nodes get returned properly
        - testing2 (x,y) plane
        '''
        value = self.mp( 0.225 )
        self.assertEqual( value, 0.4475 ) 
        
if __name__ == '__main__':
    unittest.main()



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Jan 19, 2010 by: rch


from traits.api import Array, Bool, Enum, Float, HasTraits, \
                                 Instance, Int, Trait, Str, Enum, \
                                 Callable, List, TraitDict, Any, Range, \
                                 Delegate, Event, on_trait_change, Button, \
                                 Interface, implements, Property, cached_property

class SimOut( HasTraits ):
    '''
    Specification of a parameter to be included in a parametric study.
    
    This information can be seen in analogy with the Item class specifying
    the components of a view. The SimOut can suggest the default value
    and the available range of values to be included in the parametric study.
    The particular setting is then given in the SimPStudy class. 
    '''
    
    name = Str
    
    unit = Str('?')
    
    # for numerical values
    